@@ -17,7 +17,7 @@ const char	*str;
 #if	!SILENT
 	set_color(2);		/* set color to red */
 	printf("%s\n", str);
-	default_color();
+	default_color();	/* restore to default color */
 #endif
 }
 
@@ -59,6 +59,12 @@ error_bug(str)
 const char	*str;	/* constant string to display */
 {
 	error(str);		/* Display the passed error message. */
+#if	SILENT
+	printf("%s\n", str);
+#endif
+	printf(_("Please report this bug to the maintainers,\n"));
+	printf(_("along with the entry sequence that caused it.\n"));
+	printf(_("Type \"help bugs\" for info on how to report bugs found in this program.\n"));
 	longjmp(jmp_save, 13);	/* Abort the current operation with the critical error number 13. */
 }
 
@@ -93,7 +99,11 @@ check_err()
  * Each has "n_tokens" elements.
  * "n_tokens" must not change until Mathomatic terminates.
  *
- * Return true if successful.
+ * init_mem() is only called once upon Mathomatic startup
+ * before using the symbolic math engine,
+ * and can be undone by calling free_mem() below.
+ *
+ * Returns true if successful, otherwise Mathomatic cannot be used.
  */
 int
 init_mem()
@@ -107,9 +117,46 @@ init_mem()
 	if (alloc_next_espace() < 0) {	/* make sure there is at least 1 equation space */
 		return false;
 	}
+	clear_all();
 	return true;
 }
 
+#if	false
+/*
+ * Free the global expression storage arrays.
+ * After calling this, memory usage is reset and Mathomatic becomes unusable,
+ * so do not call unless finished with using the Mathomatic code.
+ *
+ * This routine is not needed, because when a program exits,
+ * all the memory it allocated is released by the operating system.
+ * Inclusion of this routine was requested by Tam Hanna.
+ */
+void
+free_mem()
+{
+	int i;
+
+	clear_all();
+
+	free(scratch);
+	free(tes);
+	free(tlhs);
+	free(trhs);
+
+	for (i = 0; i < N_EQUATIONS; i++) {
+		if (lhs[i]) {
+			free(lhs[i]);
+			lhs[i] = NULL;
+		}
+		if (rhs[i]) {
+			free(rhs[i]);
+			rhs[i] = NULL;
+		}
+	}
+	n_equations = 0;
+}
+#endif
+
 /*
  * Initialize some important global variables to their defaults.
  * This is called on startup and by process() to reset the global flags to the default state.
@@ -377,6 +424,8 @@ long	v;	/* standard Mathomatic variable */
 		return false;
 	if (found_var(lhs[i], n_lhs[i], v))
 		return true;
+	if (n_rhs[i] <= 0)
+		return false;
 	if (found_var(rhs[i], n_rhs[i], v))
 		return true;
 	return false;
@@ -443,6 +492,7 @@ int		n;		/* expression length */
 /*
  * Get default equation number from a command parameter string.
  * The equation number must be the only parameter.
+ * If no equation number is specified, default to the current equation.
  *
  * Return -1 on error.
  */
@@ -480,7 +530,7 @@ int		*np;		/* pointer to the returned parsed expression length */
 	char	*cp;
 
 #if	LIBRARY
-	snprintf(buf, sizeof(buf), "#%+d", pull_number + 1);
+	snprintf(buf, sizeof(buf), "#%+d", pull_number);
 	pull_number++;
 	cp = parse_expr(equation, np, buf);
 	return(cp && *np > 0);
@@ -569,7 +619,7 @@ current_not_defined()
  *
  * Returns "string" if successful or NULL on error.
  */
-char	*
+char *
 get_string(string, n)
 char	*string;	/* storage for input string */
 int	n;		/* maximum size of "string" in bytes */
@@ -664,7 +714,9 @@ get_yes_no()
 }
 
 /*
- * Store or display the result of a command.
+ * Display the result of a command,
+ * or store the pointer to the text of the listed expression
+ * into result_str if compiled for the library.
  *
  * Return true if successful.
  */
@@ -681,7 +733,7 @@ int	en;	/* equation space number the result is in */
 		if (factor_int_flag) {
 			factor_int_sub(en);
 		}
-		free(result_str);
+		free_result_str();
 		result_str = list_equation(en, false);
 		return(result_str != NULL);
 	}
@@ -689,6 +741,19 @@ int	en;	/* equation space number the result is in */
 	return(list_sub(en) != 0);
 }
 
+/*
+ * Free any malloc()ed result_str, so there won't be a memory leak
+ * in the symbolic math library.
+ */
+void
+free_result_str()
+{
+	if (result_str) {
+		free(result_str);
+		result_str = NULL;
+	}
+}
+
 /*
  * Return true if the first word in the passed string is "all".
  */
@@ -782,7 +847,7 @@ char	*cp;	/* command line string */
 	if (cp) {
 		cp = skip_space(cp);
 		if (*cp) {
-			error(_("Extra characters encountered."));
+			error(_("Extra characters or unrecognized argument."));
 			return true;
 		}
 	}
@@ -810,7 +875,7 @@ int	*ip, *jp;
 /*
  * Skip over space characters.
  */
-char	*
+char *
 skip_space(cp)
 char	*cp;	/* character pointer */
 {
@@ -839,21 +904,31 @@ char	*cp, **cpp;
 }
 
 /*
- * Skip over a parameter (command line argument) in a character string.
- * Parameters are separated with spaces.
- *
- * Returns a string pointer to the next parameter.
+ * Return true if passed character is a Mathomatic command parameter delimiter.
  */
-char	*
+int
+isdelimiter(ch)
+int	ch;
+{
+	return(isspace(ch) || ch == ',' || ch == '=');
+}
+
+/*
+ * Skip over the current parameter in a Mathomatic command line string.
+ * Parameters are usually separated with spaces.
+ *
+ * Returns a string (character pointer) to the next parameter.
+ */
+char *
 skip_param(cp)
-char	*cp;	/* character pointer */
+char	*cp;
 {
 	if (cp) {
-		while (*cp && (!isascii(*cp) || (!isspace(*cp) && *cp != '='))) {
+		while (*cp && (!isascii(*cp) || !isdelimiter(*cp))) {
 			cp++;
 		}
 		cp = skip_space(cp);
-		if (*cp == '=') {
+		if (isdelimiter(*cp)) {
 			cp = skip_space(cp + 1);
 		}
 	}
@@ -861,11 +936,11 @@ char	*cp;	/* character pointer */
 }
 
 /*
- * Compare strings up to the end or the first space.
+ * Compare strings up to the end or the first space or parameter delimiter.
  * Must be an exact match.
  * Compare is alphabetic case insensitive.
  *
- * Returns zero on match, non-zero if different.
+ * Returns zero on exact match, otherwise non-zero if strings are different.
  */
 int
 strcmp_tospace(cp1, cp2)
@@ -873,9 +948,9 @@ char	*cp1, *cp2;
 {
 	char	*cp1a, *cp2a;
 
-	for (cp1a = cp1; *cp1a && !isspace(*cp1a); cp1a++)
+	for (cp1a = cp1; *cp1a && !isdelimiter(*cp1a); cp1a++)
 		;
-	for (cp2a = cp2; *cp2a && !isspace(*cp2a); cp2a++)
+	for (cp2a = cp2; *cp2a && !isdelimiter(*cp2a); cp2a++)
 		;
 	return strncasecmp(cp1, cp2, max(cp1a - cp1, cp2a - cp2));
 }
@@ -887,12 +962,11 @@ int
 level_plus_count(p1, n1, level)
 token_type	*p1;	/* expression pointer */
 int		n1;	/* expression length */
-int		level;	/* parentheses level number */
+int		level;	/* parentheses level number to check */
 {
 	int	i;
-	int	count;
+	int	count = 0;
 
-	count = 0;
 	for (i = 1; i < n1; i += 2) {
 		if (p1[i].level == level) {
 			switch (p1[i].token.operatr) {
@@ -925,9 +999,8 @@ token_type	*p1;	/* expression pointer */
 int		n1;	/* expression length */
 {
 	int	i;
-	int	count;
+	int	count = 0;
 
-	count = 0;
 	for (i = 0; i < n1; i += 2) {
 		if (p1[i].kind == VARIABLE) {
 			count++;
@@ -981,7 +1054,7 @@ int		n1;	/* expression length */
 {
 	int	i;
 
-	for (i = 0; i < n1; i += 2) {
+	for (i = 0; i < n1; i++) {
 		if (p1[i].kind == CONSTANT && !isfinite(p1[i].token.constant)) {
 			return true;
 		}
@@ -1028,14 +1101,16 @@ int		n1;	/* expression length */
 
 /*
  * Check for division by zero.
+ *
+ * Display a warning and return true if passed double is 0.
  */
-void
+int
 check_divide_by_zero(denominator)
 double	denominator;
 {
-#if	!TRAP_FPERRORS
 	if (denominator == 0) {
 		warning(_("Division by zero."));
+		return true;
 	}
-#endif
+	return false;
 }
