@@ -15,22 +15,25 @@
 #define INFINITY	HUGE_VAL			/* the floating point, positive infinity constant */
 #endif
 #ifndef	isfinite
-#define	isfinite(d)	finite(d)			/* true if d is finite (not infinity or NaN) */
+#define	isfinite(d)	finite(d)			/* true if d is finite (not infinity nor NaN) */
+#endif
+
+#ifndef PATH_MAX					/* make sure the maximum filename length is set */
+#define PATH_MAX	_POSIX_PATH_MAX
 #endif
 
 #define	ARR_CNT(a)	((int) (sizeof(a)/sizeof(a[0])))	/* returns the number of elements in an array */
 #define CLEAR_ARRAY(a)	memset(a, 0, sizeof(a))			/* sets all elements of an array to zero */
 
-#define	MAX_K_INTEGER	1.0e14				/* maximum representable integer, 14 digits for doubles */
-#define	MAX_PRECISION	16				/* maximum useful display precision (number of digits) */
+#define	MAX_K_INTEGER	1.0e15		/* maximum safely representable floating point integer, 15 digits for doubles */
 
 #define	blt(dst, src, cnt)	memmove((char *) (dst), (char *) (src), (size_t) (cnt))	/* memory copy function */
 #define always_positive(power)	(fmod((double) (power), 2.0) == 0.0)	/* true if all real numbers raised to "power" result in positive, real numbers */
 
-#if	I18N		/* internationalization: output in languages other than English if strings are translated */
-#define _(str)		gettext(str)	/* currently incomplete and untested */
+#if	I18N		/* internationalization: output in languages other than English using gettext(3) */
+#define _(str)		gettext(str)
 #else
-#define _(str)		str
+#define _(str)		str		/* constant strings to be translated should be marked with this macro */
 #endif
 
 #define	STANDARD_SCREEN_COLUMNS	80			/* default number of columns of characters on the screen */
@@ -38,10 +41,10 @@
 
 #define	TMP_FILE	"/tmp/mathomatic.XXXXXX"	/* temporary file template for mkstemp(3) */
 
-#define	PROMPT		"-> "				/* user interface prompt string */
-#define	HTML_PROMPT	"&mdash;&gt; "
+#define	PROMPT_STR	"-> "				/* user interface main prompt strings, preceded by the current equation number */
+#define	HTML_PROMPT_STR	"&mdash;&gt; "
 
-#define	MAX_CMD_LEN	(max(PATH_MAX, 500))		/* maximum command line length (not expression input), also max filename length */
+#define	MAX_CMD_LEN	(max(PATH_MAX, 512))		/* maximum command line length (not expression input), also max filename length */
 #define	MAX_PROMPT_LEN	STANDARD_SCREEN_COLUMNS		/* maximum length of prompts */
 
 /*
@@ -50,7 +53,7 @@
  */
 #ifndef	N_EQUATIONS
 #if	HANDHELD
-#define	N_EQUATIONS	20
+#define	N_EQUATIONS	50
 #else
 #define	N_EQUATIONS	100
 #endif
@@ -66,9 +69,9 @@
  */
 #ifndef	DEFAULT_N_TOKENS
 #if	HANDHELD
-#define	DEFAULT_N_TOKENS	300
+#define	DEFAULT_N_TOKENS	6000
 #else
-#define	DEFAULT_N_TOKENS	30000
+#define	DEFAULT_N_TOKENS	60000
 #endif
 #endif
 
