@@ -6,17 +6,72 @@
 
 #include "includes.h"
 
-#define	OPT_MIN_SIZE	7	/* Minimum size of repeated expressions to find in optimize command. */
+#define	OPT_MIN_SIZE	7	/* Minimum size (in tokens) of repeated expressions to find in optimize command. */
 
 static int	sum_product();
 static int	complex_func();
 static int	elim_sub();
-#if	!LIBRARY
-static int	edit_sub();
 
 /* Global variables for the optimize command. */
 static int	opt_en[N_EQUATIONS];
 static int	last_temp_var = 0;
+
+#if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
+/*
+ * The plot command.
+ */
+int
+plot_cmd(cp)
+char	*cp;	/* the command line argument */
+{
+	char	cl[MAX_CMD_LEN];
+	int	free_cp = false;
+
+	if (security_level) {
+		error(_("Command disabled."));
+		return false;
+	}
+	if (*cp == '\0') {
+		if (current_not_defined()) {
+			return false;
+		}
+		if (n_rhs[cur_equation]) {
+			cp = list_expression(rhs[cur_equation], n_rhs[cur_equation], 3);
+		} else {
+			cp = list_expression(lhs[cur_equation], n_lhs[cur_equation], 3);
+		}
+		if (cp) {
+			free_cp = true;
+		} else {
+			return false;
+		}
+	}
+	if (strpbrk(cp, "'\\")) {
+		error(_("Invalid character in expression string."));
+		if (free_cp)
+			free(cp);
+		return false;
+	}
+	if (strchr(cp, 'y')) {
+		printf(_("Performing 3D surface plot...\n"));
+		snprintf(cl, sizeof(cl), "echo 'splot %s'|gnuplot -persist", cp);
+	} else {
+		printf(_("Performing 2D plot...\n"));
+		snprintf(cl, sizeof(cl), "echo 'plot %s'|gnuplot -persist", cp);
+	}
+	if (free_cp)
+		free(cp);
+	if (shell_out(cl)) {
+		error(_("Error trying to run gnuplot."));
+		printf(_("Command line = \"%s\".\n"), cl);
+		return false;
+#if	!SILENT
+	} else if (debug_level > 0) {
+		printf(_("Command line = \"%s\".\n"), cl);
+#endif
+	}
+	return true;
+}
 #endif
 
 /*
@@ -28,23 +83,29 @@ char	*cp;	/* the command line argument */
 {
 	int		rv;	/* return value */
 
-	extra_characters(cp);	/* no command options allowed */
 #if	LIBRARY
-	free(result_str);
+	free_result_str();
 	result_str = strdup(VERSION);
 #endif
 	rv = version_report();
+
+	fprintf(gfp, _("\nMathomatic is GNU LGPL version 2.1 licensed software,\n"));
+	fprintf(gfp, _("meaning it is free software that comes with no warranty.\n"));
+	fprintf(gfp, _("Type \"help copyright\" for the copyright and license.\n"));
+
+	fprintf(gfp, _("\nThe newest version of Mathomatic is always available at\n"));
+	fprintf(gfp, _("the Mathomatic website: http://mathomatic.org\n"));
 	return rv;
 }
 
 /*
- * Report version number, compile flags, and memory usage.
+ * Display version info.
  */
 int
 version_report()
 {
 	fprintf(gfp, _("Mathomatic version %s\n"), VERSION);
-	fprintf(gfp, _("Compile flags used: "));
+	fprintf(gfp, _("Compile-time defines used: "));
 #if	UNIX
 	fprintf(gfp, "UNIX ");
 #endif
@@ -72,14 +133,24 @@ version_report()
 #if	I18N
 	fprintf(gfp, "I18N ");
 #endif
-	fprintf(gfp, _("\nThe current expression array size is %d tokens,\n"), n_tokens);
-	fprintf(gfp, _("maximum memory usage: %ld kilobytes.\n"), ((long) (N_EQUATIONS + 3L) * n_tokens * sizeof(token_type) * 2L) / 1000L);
-	fprintf(gfp, _("This is GNU LGPL version 2.1 licensed software.\n"));
+#if	NO_COLOR
+	fprintf(gfp, "NO_COLOR ");
+#endif
+	fprintf(gfp, _("\n\nThe current expression array size is %d tokens,\n"), n_tokens);
+	fprintf(gfp, _("making the maximum memory usage %ld kilobytes.\n"), ((long) (N_EQUATIONS + 3L) * n_tokens * sizeof(token_type) * 2L) / 1000L);
+#if	SECURE
+	fprintf(gfp, _("Compiled for maximum security.\n"));
+#else
+	fprintf(gfp, _("The current security level is %d.\n"), security_level);
+#endif
 	return true;
 }
 
 /*
  * The solve command.
+ *
+ * Return 0 on failure, 1 on total success,
+ * or 2 if partial success (solutions didn't totally verify when doing "solve verify").
  */
 int
 solve_cmd(cp)
@@ -90,7 +161,8 @@ char	*cp;
 	long		v;	/* Mathomatic variable */
 	token_type	want;
 	int		diff_sign;
-	int		verify_flag = false, plural_flag = false, once_through;
+	int		verify_flag = false, plural_flag = false, once_through, contains_infinity;
+	char		*cp1;
 
 	if (cur_equation < 0 || cur_equation >= n_equations || n_lhs[cur_equation] <= 0 || n_rhs[cur_equation] <= 0) {
 		error(_("No current equation."));
@@ -101,6 +173,12 @@ char	*cp;
 		verify_flag = true;
 		cp = skip_param(cp);
 	}
+	if (strcmp_tospace(cp, "for") == 0) {
+		cp1 = skip_param(cp);
+		if (*cp1) {
+			cp = cp1;
+		}
+	}
 	if (*cp == '\0') {
 		my_strlcpy(prompt_str, _("Enter variable to solve for: "), sizeof(prompt_str));
 		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
@@ -119,7 +197,7 @@ char	*cp;
 		if (solve_sub(&want, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation]) > 0) {
 			return_result(cur_equation);
 			if (compare_es(cur_equation, i)) {
-				warning(_("Nothing was done, so no need to verify."));
+				debug_string(0, _("Nothing was done, so no need to verify solution."));
 				n_lhs[i] = 0;
 				n_rhs[i] = 0;
 				return true;
@@ -136,27 +214,31 @@ char	*cp;
 			calc_simp(rhs[i], &n_rhs[i]);
 			once_through = false;
 check_result:
-			if (se_compare(lhs[i], n_lhs[i], rhs[i], n_rhs[i], &diff_sign) && !diff_sign) {
-#if	!SILENT
+			contains_infinity = (exp_contains_infinity(lhs[i], n_lhs[i])
+			    || exp_contains_infinity(rhs[i], n_rhs[i]));
+                        if (!contains_infinity && se_compare(lhs[i], n_lhs[i], rhs[i], n_rhs[i], &diff_sign) && !diff_sign) {
 				if (plural_flag)
 					printf(_("Solutions verified.\n"));
 				else
 					printf(_("Solution verified.\n"));
-#endif
 				n_lhs[i] = 0;
 				n_rhs[i] = 0;
 				return true;
 			} else {
-				if (!once_through) {
-					symb_flag = true;
+				if (!contains_infinity && !once_through) {
+					symb_flag = symblify;
 					simpa_side(lhs[i], &n_lhs[i], true, false);
 					simpa_side(rhs[i], &n_rhs[i], true, false);
 					symb_flag = false;
 					once_through = true;
 					goto check_result;
 				}
-				error(_("Solution may be incorrect."));
-				return false;
+				error(_("Solution might be incorrect."));
+#if	true	/* set to false for testing */
+				n_lhs[i] = 0;
+				n_rhs[i] = 0;
+#endif
+				return 2;
 			}
 		} else {
 #if	!SILENT
@@ -257,7 +339,7 @@ int	product_flag;	/* true for product, otherwise sum */
 	}
 	start = strtod(cp1, &cp);
 	if (cp1 == cp) {
-		error(_("Invalid number."));
+		error(_("Number expected."));
 		return false;
 	}
 	if (fabs(start) >= MAX_K_INTEGER) {
@@ -277,7 +359,7 @@ int	product_flag;	/* true for product, otherwise sum */
 	}
 	end = strtod(cp1, &cp);
 	if (cp1 == cp) {
-		error(_("Invalid number."));
+		error(_("Number expected."));
 		return false;
 	}
 	if (fabs(end) >= MAX_K_INTEGER) {
@@ -345,7 +427,6 @@ int	product_flag;	/* true for product, otherwise sum */
 	return return_result(result_en);
 }
 
-#if	!LIBRARY
 /*
  * This function is for the "optimize" command.
  * It finds and substitutes all occurrences of the RHS of "en" in "equation".
@@ -448,7 +529,7 @@ int		*np;
 					k1 = i1 - 1;
 					if ((jj1 - k1) >= OPT_MIN_SIZE
 					    && se_compare(&equation[k], j - k, &equation[k1], jj1 - k1, &diff_sign)) {
-						snprintf(var_name_buf, sizeof(var_name_buf), "temp%d", last_temp_var);
+						snprintf(var_name_buf, sizeof(var_name_buf), "temp%.0d", last_temp_var);
 						if (parse_var(&v, var_name_buf) == NULL) {
 							return false;	/* can't create "temp" variable */
 						}
@@ -587,7 +668,6 @@ char	*cp;
 	}
 	return rv;
 }
-#endif
 
 #if	READLINE
 /*
@@ -600,7 +680,7 @@ char	*cp;
 	int	i, j;
 
 	if (!readline_enabled) {
-		error(_("Readline is disabled."));
+		error(_("Readline is currently turned off."));
 		return false;
 	}
 	if (!get_range_eol(&cp, &i, &j)) {
@@ -625,10 +705,12 @@ char	*cp;
  */
 int
 push_en(en)
-int	en;	/* equation space number */
+int	en;	/* equation space number to push */
 {
 	char	*cp;
 
+	if (!readline_enabled)
+		return false;
 	high_prec = true;
 	cp = list_equation(en, false);
 	high_prec = false;
@@ -641,18 +723,19 @@ int	en;	/* equation space number */
 #endif
 
 /*
- * Display current working directory.
+ * Output the current working directory.
  *
  * Return true if successful.
  */
 int
-display_current_directory()
+output_current_directory(ofp)
+FILE	*ofp;	/* output file pointer */
 {
 #if	!SECURE
-	char	buf[MAX_CMD_LEN];
+	char	buf[PATH_MAX];
 
-	if (getcwd(buf, sizeof(buf))) {
-		fprintf(gfp, "directory = %s\n", buf);
+	if (security_level < 3 && ofp && getcwd(buf, sizeof(buf))) {
+		fprintf(ofp, "directory = %s\n", buf);
 		return true;
 	}
 #endif
@@ -664,8 +747,11 @@ display_current_directory()
  */
 void
 output_options(ofp)
-FILE	*ofp;	/* Output file pointer */
+FILE	*ofp;	/* output file pointer */
 {
+	if (ofp == NULL)
+		return;
+
 	fprintf(ofp, "precision = %d digits\n", precision);
 
 	if (!autosolve) {
@@ -678,6 +764,11 @@ FILE	*ofp;	/* Output file pointer */
 	}
 	fprintf(ofp, "autocalc\n");
 
+	if (!autoselect) {
+		fprintf(ofp, "no ");
+	}
+	fprintf(ofp, "autoselect\n");
+
 #if	!SILENT
 	fprintf(ofp, "debug_level = %d\n", debug_level);
 #endif
@@ -712,6 +803,11 @@ FILE	*ofp;	/* Output file pointer */
 	}
 	fprintf(ofp, "preserve_surds\n");
 
+	if (!rationalize_denominators) {
+		fprintf(ofp, "no ");
+	}
+	fprintf(ofp, "rationalize_denominators\n");
+
 	fprintf(ofp, "modulus_mode = %d\n", modulus_mode);
 
 	fprintf(ofp, "finance = %d\n", finance_option);
@@ -756,6 +852,38 @@ char	**cpp;
 	return false;
 }
 
+#if	!SECURE && !LIBRARY
+/*
+ * Save set options in the startup file, displaying a confirmation message.
+ *
+ * Return true if successful.
+ */
+int
+save_set_options()
+{
+	FILE	*fp;
+
+	if (rc_file[0] == '\0') {
+		error(_("Set options startup file name not set."));
+		return false;
+	}
+	if ((fp = fopen(rc_file, "w")) == NULL) {
+		error(_("Unable to write to set options startup file."));
+		return false;
+	}
+	fprintf(fp, "; Mathomatic set options loaded at startup.\n");
+	fprintf(fp, "; This file can be edited.\n\n");
+	output_options(fp);
+	if (fclose(fp) == 0) {
+		printf(_("All options saved in startup file \"%s\".\n"), rc_file);
+	} else {
+		error(_("Error saving set options."));
+		return false;
+	}
+	return true;
+}
+#endif
+
 /*
  * Handle parsing of options for the set command.
  *
@@ -767,14 +895,15 @@ char	*cp;
 {
 	int	i;
 	int	negate;
-	char	*cp1;
+	char	*cp1, *option_string;
 
+try_next_param:
 	cp = skip_space(cp);
 	if (*cp == '\0') {
 		return true;
 	}
-#if	!SECURE
-	if (strncasecmp(cp, "dir", 3) == 0) {
+#if	!SECURE && !LIBRARY
+	if (security_level < 3 && strncasecmp(cp, "dir", 3) == 0) {
 		cp = skip_param(cp);
 		if (*cp == '\0') {
 			cp1 = getenv("HOME");
@@ -786,14 +915,15 @@ char	*cp;
 			error(_("Error changing directory."));
 			return false;
 		}
-		display_current_directory();
+		output_current_directory(stdout);
 		return true;
 	}
 #endif
 	negate = skip_no(&cp);
+	option_string = cp;
+	cp = skip_param(cp);
 #if	!SILENT
-	if (strncasecmp(cp, "debug", 5) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "debug", 5) == 0) {
 		if (negate) {
 			debug_level = 0;
 		} else {
@@ -808,8 +938,7 @@ char	*cp;
 		goto check_return;
 	}
 #endif
-	if (strncasecmp(cp, "special", 7) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "special", 7) == 0) {
 		if (negate) {
 			special_variable_characters[0] = '\0';
 		} else {
@@ -817,8 +946,7 @@ char	*cp;
 		}
 		return true;
 	}
-	if (strncasecmp(cp, "columns", 7) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "columns", 6) == 0) {
 		if (negate) {
 			screen_columns = 0;
 		} else {
@@ -832,8 +960,21 @@ char	*cp;
 		}
 		goto check_return;
 	}
-	if (strncasecmp(cp, "precision", 4) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "wide", 4) == 0) {
+		if (!negate) {
+			screen_columns = 0;
+			screen_rows = 0;
+		} else {
+#if	UNIX || CYGWIN
+			get_screen_size();
+#else
+			screen_columns = STANDARD_SCREEN_COLUMNS;
+			screen_rows = STANDARD_SCREEN_ROWS;
+#endif
+		}
+		goto try_next_param;
+	}
+	if (strncasecmp(option_string, "precision", 4) == 0) {
 		i = decstrtol(cp, &cp1);
 		if (i < 0 || i > 14 || cp == cp1) {
 			error(_("Please specify a display precision between 0 and 14 digits."));
@@ -842,38 +983,43 @@ char	*cp;
 		precision = i;
 		return true;
 	}
-	if (strncasecmp(cp, "autosolve", 9) == 0) {
-		cp = skip_param(cp);
+	if (strcmp_tospace(option_string, "auto") == 0) {
+		autosolve = autocalc = autoselect = !negate;
+		goto try_next_param;
+	}
+	if (strncasecmp(option_string, "autosolve", 9) == 0) {
 		autosolve = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
-	if (strncasecmp(cp, "autocalc", 8) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "autocalc", 8) == 0) {
 		autocalc = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
-	if (strncasecmp(cp, "case", 4) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "autoselect", 10) == 0) {
+		autoselect = !negate;
+		goto try_next_param;
+	}
+	if (strncasecmp(option_string, "case", 4) == 0) {
 		case_sensitive_flag = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
-	if (strncasecmp(cp, "display2d", 7) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "display2d", 7) == 0) {
 		display2d = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
-	if (strncasecmp(cp, "prompt", 6) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "prompt", 6) == 0) {
 		quiet_mode = negate;
-		goto check_return;
+		goto try_next_param;
 	}
-	if (strncasecmp(cp, "preserve", 8) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "preserve", 8) == 0) {
 		preserve_surds = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
-	if (strncasecmp(cp, "modulus_mode", 3) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "rationalize", 11) == 0) {
+		rationalize_denominators = !negate;
+		goto try_next_param;
+	}
+	if (strncasecmp(option_string, "modulus_mode", 3) == 0) {
 		if (negate) {
 			modulus_mode = 0;
 		} else {
@@ -891,20 +1037,17 @@ char	*cp;
 		}
 		goto check_return;
 	}
-	if (strncasecmp(cp, "color", 5) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "color", 5) == 0) {
+		reset_attr();
 		color_flag = !negate;
-		cur_color = -1;
-		goto check_return;
+		goto try_next_param;
 	}
-	if (strncasecmp(cp, "bold", 4) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "bold", 4) == 0) {
+		reset_attr();
 		bold_colors = !negate;
-		cur_color = -1;
-		goto check_return;
+		goto try_next_param;
 	}
-	if (strncasecmp(cp, "finance", 7) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "finance", 7) == 0) {
 		if (negate) {
 			finance_option = 0;
 		} else {
@@ -926,21 +1069,35 @@ char	*cp;
 		}
 		goto check_return;
 	}
-	if (strncasecmp(cp, "factor_integers", 6) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "factor_integers", 6) == 0) {
 		factor_int_flag = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
-	if (strncasecmp(cp, "right_associative_power", 5) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "right_associative_power", 5) == 0) {
 		right_associative_power = !negate;
-		goto check_return;
+		goto try_next_param;
 	}
-	if (strncasecmp(cp, "negate_highest_precedence", 6) == 0) {
-		cp = skip_param(cp);
+	if (strncasecmp(option_string, "negate_highest_precedence", 6) == 0) {
 		negate_highest_precedence = !negate;
+		goto try_next_param;
+	}
+#if	!SECURE && !LIBRARY
+	if (security_level < 2 && strcmp_tospace(option_string, "save") == 0) {
+		if (rc_file[0] == '\0') {
+			error(_("Set options startup file name not set."));
+			return false;
+		}
+		if (negate) {
+			if (unlink(rc_file) == 0) {
+				printf(_("Set options startup file \"%s\" removed.\n"), rc_file);
+			}
+		} else {
+			if (!save_set_options())
+				return false;
+		}
 		goto check_return;
 	}
+#endif
 	error(_("Unknown set option."));
 	return false;
 
@@ -964,7 +1121,7 @@ char	*cp;
 
 		fprintf(gfp, "columns = %d\n", screen_columns);
 
-		display_current_directory();
+		output_current_directory(gfp);
 		return true;
 	}
 	return set_options(cp);
@@ -1235,9 +1392,6 @@ char	*cp;
 #if	!LIBRARY
 /*
  * The calculate command.
- *
- * Temporarily plug values into the RHS of the current equation,
- * unless there is no RHS, then use LHS.
  */
 int
 calculate_cmd(cp)
@@ -1429,7 +1583,7 @@ calc_again:
 			fprintf(gfp, " = ");
 		}
 		list_factor(tlhs, &n_tlhs, factor_flag);
-		if (make_fractions(tlhs, &n_tlhs)) {
+		if (n_tlhs <= 9 && make_fractions(tlhs, &n_tlhs)) {
 			group_proc(tlhs, &n_tlhs);
 			fprintf(gfp, " = ");
 			list_factor(tlhs, &n_tlhs, factor_flag);
@@ -1568,8 +1722,8 @@ char	*cp;
 			}
 			debug_string(0, _("Simplifying both expressions..."));
 			symb_flag = symb;
-			simpa_side(lhs[i], &n_lhs[i], false, false);
-			simpa_side(lhs[j], &n_lhs[j], false, false);
+			simpa_side(lhs[i], &n_lhs[i], false, true);
+			simpa_side(lhs[j], &n_lhs[j], false, true);
 			symb_flag = false;
 #if	!SILENT
 			if (debug_level >= 0) {
@@ -1605,8 +1759,8 @@ char	*cp;
 		}
 		debug_string(0, _("Simplifying both equations..."));
 		symb_flag = symb;
-		simpa_side(rhs[i], &n_rhs[i], false, false);
-		simpa_side(rhs[j], &n_rhs[j], false, false);
+		simpa_side(rhs[i], &n_rhs[i], false, true);
+		simpa_side(rhs[j], &n_rhs[j], false, true);
 		symb_flag = false;
 #if	!SILENT
 		if (debug_level >= 0) {
@@ -1668,10 +1822,12 @@ times_neg1:
 	}
 	list_proc(lhs[j], n_lhs[j], false);
 	fprintf(gfp, _(") in the second equation.\n"));
-	return(!diff_sign);
+	if (diff_sign)
+		return 0;	/* equations not identical */
+	else
+		return 2;	/* equations identical, but LHS variable name differs */
 }
 
-#if	!LIBRARY
 /*
  * The divide command.
  */
@@ -1681,10 +1837,11 @@ char	*cp;
 {
 	long		v = 0, v_tmp;		/* Mathomatic variables */
 	int		i, j;
-	int		nl, nr;
+	int		nleft, nright;
 	double		lcm, d1, d2, d3, d4, d5;
 	complexs	c1, c2, c3;
 
+	pull_number = -1;	/* Operands are last two entered expressions when using library. */
 	if (*cp) {
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
@@ -1694,32 +1851,32 @@ char	*cp;
 			return false;
 	}
 	i = next_espace();
+/* prompt for the two operands */
 	my_strlcpy(prompt_str, _("Enter dividend: "), sizeof(prompt_str));
-	if (!get_expr(rhs[i], &nr)) {
+	if (!get_expr(rhs[i], &nright)) {
 		return false;
 	}
 	my_strlcpy(prompt_str, _("Enter divisor: "), sizeof(prompt_str));
-	if (!get_expr(lhs[i], &nl)) {
+	if (!get_expr(lhs[i], &nleft)) {
 		return false;
 	}
 	fprintf(gfp, "\n");
-/* simplify the input expressions */
-	calc_simp(rhs[i], &nr);
-	calc_simp(lhs[i], &nl);
+/* simplify the operand expressions */
+	calc_simp(rhs[i], &nright);
+	calc_simp(lhs[i], &nleft);
 /* if division by zero, display a warning */
-	if (get_constant(lhs[i], nl, &d2)) {
+	if (get_constant(lhs[i], nleft, &d2)) {
 		check_divide_by_zero(d2);
 	}
 /* Do constant division if 2 normal numbers were entered */
-	if (get_constant(rhs[i], nr, &d1) && get_constant(lhs[i], nl, &d2)) {
+	if (get_constant(rhs[i], nright, &d1) && get_constant(lhs[i], nleft, &d2)) {
 		d3 = gcd_verified(d1, d2);
 		d4 = modf(d1 / d2, &d5);
-		if (d3 == 0 || d3 == 1) {
-			fprintf(gfp, "%.*g/%.*g = %.*g\n", precision, d1, precision, d2, precision, d1 / d2);
-		} else {
-			fprintf(gfp, "%.*g/%.*g = %.*g/%.*g = %.*g\n", precision, d1, precision, d2, precision, d1 / d3, precision, d2 / d3, precision, d1 / d2);
+		fprintf(gfp, "%.*g/%.*g = %.*g", precision, d1, precision, d2, precision, d1 / d2);
+		if (d3 != 0.0 && (d2 / d3) != 1.0) {
+			fprintf(gfp, " = %.*g/%.*g", precision, d1 / d3, precision, d2 / d3);
 		}
-		fprintf(gfp, _("Quotient: %.*g, Remainder: %.*g\n"), precision, d5, precision, d4 * d2);
+		fprintf(gfp, _("\nQuotient: %.*g, Remainder: %.*g\n"), precision, d5, precision, d4 * d2);
 		d1 = fabs(d1);
 		d2 = fabs(d2);
 		if (d3 == 0.0) {
@@ -1727,22 +1884,22 @@ char	*cp;
 			return true;
 		}
 		fprintf(gfp, "GCD = ");
-		if (factor_one(d3) && !is_prime()) {
+		if (d3 >= 4.0 && factor_one(d3) && !is_prime()) {
 			display_unique();
 		} else {
-			fprintf(gfp, "%.*g\n", precision, d3);
+			display_fraction(d3);
 		}
 		lcm = (d1 * d2) / d3;
 		fprintf(gfp, "LCM = ");
-		if (factor_one(lcm) && !is_prime()) {
+		if (lcm >= 4.0 && factor_one(lcm) && !is_prime()) {
 			display_unique();
 		} else {
-			fprintf(gfp, "%.*g\n", precision, lcm);
+			display_fraction(lcm);
 		}
 		return true;
 	}
 /* else do complex number division if 2 complex numbers were entered */
-	if (parse_complex(rhs[i], nr, &c1) && parse_complex(lhs[i], nl, &c2)) {
+	if (parse_complex(rhs[i], nright, &c1) && parse_complex(lhs[i], nleft, &c2)) {
 		c3 = complex_div(c1, c2);
 		fprintf(gfp, _("Result of complex number division:\n"));
 		fprintf(gfp, "%.*g %+.*g*i\n\n", precision, c3.re, precision, c3.im);
@@ -1750,7 +1907,7 @@ char	*cp;
 	}
 /* else do polynomial division and univariate GCD display */
 	v_tmp = v;
-	if (poly_div(rhs[i], nr, lhs[i], nl, &v_tmp)) {
+	if (poly_div(rhs[i], nright, lhs[i], nleft, &v_tmp)) {
 		simp_divide(tlhs, &n_tlhs);
 		simp_divide(trhs, &n_trhs);
 		list_var(v_tmp, 0);
@@ -1766,9 +1923,9 @@ char	*cp;
 		fprintf(gfp, _("Polynomial division failed.\n"));
 	}
 	fprintf(gfp, "\n");
-	j = poly_gcd(rhs[i], nr, lhs[i], nl, v);
+	j = poly_gcd(rhs[i], nright, lhs[i], nleft, v);
 	if (!j) {
-		j = poly_gcd(lhs[i], nl, rhs[i], nr, v);
+		j = poly_gcd(lhs[i], nleft, rhs[i], nright, v);
 	}
 	if (j) {
 		simp_divide(trhs, &n_trhs);
@@ -1781,7 +1938,6 @@ char	*cp;
 	}
 	return true;
 }
-#endif
 
 /*
  * The eliminate command.
@@ -1796,11 +1952,8 @@ char	*cp;
 	int	success_flag = false, did_something = false, repeat_flag = false, using_flag;
 	char	used[N_EQUATIONS];
 	char	*cp_start;
+	char	buf[MAX_CMD_LEN];
 
-	if (*cp == '\0') {
-		error(_("Please specify the variables to eliminate or \"all\" for all variables."));
-		return false;
-	}
 	CLEAR_ARRAY(used);
 	if (current_not_defined()) {
 		return false;
@@ -1809,6 +1962,13 @@ char	*cp;
 		repeat_flag = true;
 		cp = skip_param(cp);
 	}
+	if (*cp == '\0') {
+		my_strlcpy(prompt_str, _("Enter variables to eliminate: "), sizeof(prompt_str));
+		cp = get_string(buf, sizeof(buf));
+		if (cp == NULL || *cp == '\0') {
+			return false;
+		}
+	}
 	cp_start = cp;
 next_var:
 	if (vc) {
@@ -1964,7 +2124,8 @@ char	*cp;
 {
 	int	i, j;
 	char	*cp1;
-	int	factor_flag;
+	jmp_buf	save_save;
+	int	factor_flag, displayed = false;
 
 	factor_flag = (strcmp_tospace(cp, "factor") == 0);
 	if (factor_flag) {
@@ -1981,22 +2142,31 @@ char	*cp;
 		}
 		for (; i <= j; i++) {
 			if (n_lhs[i] > 0) {
+				blt(save_save, jmp_save, sizeof(jmp_save));
+				if (setjmp(jmp_save) != 0) {	/* trap errors */
+					blt(jmp_save, save_save, sizeof(jmp_save));
+					printf("Skipping equation number %d.\n", i + 1);
+					continue;
+				}
 				make_fractions_and_group(i);
 				if (factor_flag || factor_int_flag) {
 					factor_int_sub(i);
 				}
+				blt(jmp_save, save_save, sizeof(jmp_save));
 #if     LIBRARY
 	                        if (gfp == stdout) {
-	                                free(result_str);
+	                                free_result_str();
 	                                result_str = list_equation(i, false);
 	                                return(result_str != NULL);
 	                        }
 #endif
-				flist_equation(i);
+				if (flist_equation(i) > 0) {
+					displayed = true;
+				}
 			}
 		}
 	} while (*cp);
-	return true;
+	return displayed;
 }
 
 /*
@@ -2010,7 +2180,10 @@ char	*cp;
 	char	*cp1;
 	int	export_flag = 0;
 
-	if (strncasecmp(cp, "export", 4) == 0) {
+	if (strncasecmp(cp, "gnuplot", 4) == 0) {
+		export_flag = 3;
+		cp = skip_param(cp);
+	} else if (strncasecmp(cp, "export", 4) == 0) {
 		export_flag = 2;
 		cp = skip_param(cp);
 	} else if (strncasecmp(cp, "maxima", 4) == 0) {
@@ -2029,7 +2202,7 @@ char	*cp;
 		for (; i <= j; i++) {
 #if	LIBRARY
 			if (gfp == stdout) {
-				free(result_str);
+				free_result_str();
 				result_str = list_equation(i, export_flag);
 				return(result_str != NULL);
 			}
@@ -2040,7 +2213,6 @@ char	*cp;
 	return true;
 }
 
-#if	!LIBRARY
 /*
  * The code command.
  */
@@ -2136,7 +2308,7 @@ char	*cp;
 	token_type	*p1;
 	int		n1;
 	int		lang_code = 0;
-	int		int_flag = false;
+	int		int_flag = false, imag_flag = false;
 
 	if (strcmp_tospace(cp, "c") == 0 || strcmp_tospace(cp, "c++") == 0) {
 		cp = skip_param(cp);
@@ -2152,6 +2324,23 @@ char	*cp;
 	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
 	}
+	for (k = i; k <= j; k++) {
+		if (n_lhs[k] <= 0)
+			continue;
+		if (n_rhs[k] > 0) {
+			p1 = rhs[k];
+			n1 = n_rhs[k];
+		} else {
+			p1 = lhs[k];
+			n1 = n_lhs[k];
+		}
+		for (i1 = 0; i1 < n1; i1 += 2) {
+			if (p1[i1].kind == VARIABLE && p1[i1].token.variable == IMAGINARY) {
+				imag_flag = true;
+				break;
+			}
+		}
+	}
 	last_v = 0;
 	for (vc = 0;;) {
 		if (vc >= ARR_CNT(va)) {
@@ -2204,10 +2393,13 @@ char	*cp;
 		}
 		list_var(va[i1].v, lang_code);
 		if (lang_code) {
+			if (imag_flag) {
+				fprintf(gfp, "_Complex ");
+			}
 			if (int_flag) {
-				fprintf(gfp, "int\t%s;\n", var_str);
+				fprintf(gfp, "int %s;\n", var_str);
 			} else {
-				fprintf(gfp, "double\t%s;\n", var_str);
+				fprintf(gfp, "double %s;\n", var_str);
 			}
 		} else {
 			fprintf(gfp, "%s\n", var_str);
@@ -2215,7 +2407,6 @@ char	*cp;
 	}
 	return true;
 }
-#endif
 
 /*
  * The approximate command.
@@ -2242,6 +2433,10 @@ char	*cp;
 				subst_constants(rhs[i], &n_rhs[i]);
 				approximate_roots = true;
 				simp_sub(i);
+				factorv(lhs[i], &n_lhs[i], IMAGINARY);
+				if (n_rhs[i]) {
+					factorv(rhs[i], &n_rhs[i], IMAGINARY);
+				}
 				approximate_roots = false;
 				if (!return_result(i)) {
 					return false;
@@ -2380,7 +2575,7 @@ char	*cp;
 	int		k, k1;
 	long		counter, counter_max, previous_solution_number[N_EQUATIONS];
 	sign_array_type	sa_mark, sa_value;
-	int		sign_flag = false, quick_flag = false, symb = false, frac_flag = false;
+	int		sign_flag = false, quick_flag = false, quickest_flag = false, symb = false, frac_flag = false;
 
 	for (;; cp = skip_param(cp)) {
 		if (strncasecmp(cp, "sign", 4) == 0) {
@@ -2391,7 +2586,11 @@ char	*cp;
 			symb = true;
 			continue;
 		}
-		if (strncasecmp(cp, "quick", 4) == 0) {
+		if (strcmp_tospace(cp, "quickest") == 0) {
+			quickest_flag = true;
+			continue;
+		}
+		if (strcmp_tospace(cp, "quick") == 0) {
 			quick_flag = true;
 			continue;
 		}
@@ -2407,8 +2606,12 @@ char	*cp;
 	symb_flag = symb;
 	for (; i <= j; i++) {
 		if (n_lhs[i]) {
-			simpa_side(lhs[i], &n_lhs[i], quick_flag, frac_flag);
-			simpa_side(rhs[i], &n_rhs[i], quick_flag, frac_flag);
+			if (quickest_flag) {
+				simp_sub(i);
+			} else {
+				simpa_side(lhs[i], &n_lhs[i], quick_flag, frac_flag);
+				simpa_side(rhs[i], &n_rhs[i], quick_flag, frac_flag);
+			}
 			if (!return_result(i)) {
 				symb_flag = false;
 				return false;
@@ -2486,8 +2689,12 @@ char	*cp;
 				}
 				if (k)
 					fprintf(gfp, ":\n");
-				simpa_side(lhs[i1], &n_lhs[i1], quick_flag, frac_flag);
-				simpa_side(rhs[i1], &n_rhs[i1], quick_flag, frac_flag);
+				if (quickest_flag) {
+					simp_sub(i1);
+				} else {
+					simpa_side(lhs[i1], &n_lhs[i1], quick_flag, frac_flag);
+					simpa_side(rhs[i1], &n_rhs[i1], quick_flag, frac_flag);
+				}
 				for (k1 = 0; k1 < ARR_CNT(previous_solution_number); k1++) {
 					if (previous_solution_number[k1]) {
 						if (compare_es(k1, i1)) {
@@ -2518,15 +2725,19 @@ char	*cp;
 {
 	int	i, j;
 	int	i1;
+	int	found;
 	long	v;				/* Mathomatic variable */
 	double	d;
 	char	buf[MAX_CMD_LEN];
 	int	valid_range = false, repeat_flag = false, power_flag = false;
+	char	*cp1;
 
 	if (strcmp_tospace(cp, "number") == 0) {
 		cp = skip_param(cp);
 	} else if (strcmp_tospace(cp, "numbers") == 0) {
+#if	!LIBRARY
 		repeat_flag = true;
+#endif
 		cp = skip_param(cp);
 	} else {
 		if (strcmp_tospace(cp, "power") == 0) {
@@ -2534,6 +2745,16 @@ char	*cp;
 			cp = skip_param(cp);
 		}
 		valid_range = get_range(&cp, &i, &j);
+		if (!valid_range) {
+#if	LIBRARY	/* be consistent */
+			return false;
+#else		/* be helpful */
+			if (*cp == '-' || isdigit(*cp)) {
+				printf(_("Factoring integers on command line instead:\n"));
+			} else
+				return false;
+#endif
+		}
 	}
 	if (!valid_range) {
 		do {
@@ -2543,19 +2764,23 @@ char	*cp;
 				if (cp == NULL)
 					return false;
 			}
-			if (*cp == '\0') {
+			if (*cp == '\0')
 				return true;
-			}
 			for (; *cp; ) {
+				cp1 = cp;
 				d = strtod(cp, &cp);
+				if (cp == cp1) {
+					error(_("Integer expected."));
+					return false;
+				}
 				cp = skip_space(cp);
 				if (!factor_one(d)) {
-					error(_("Not a valid integer or number too large to factor."));
+					error(_("Number too large to factor or not a non-zero integer."));
 					return false;
 				}
 				display_unique();
 				if (is_prime()) {
-					fprintf(gfp, "Prime number!\n");
+					fprintf(gfp, _("Prime number!\n"));
 				}
 			}
 		} while (repeat_flag);
@@ -2581,17 +2806,31 @@ char	*cp;
 			}
 		}
 	} else {
-		v = 0;
 		do {
+			v = 0;
 			if (*cp) {
 				if ((cp = parse_var2(&v, cp)) == NULL) {
 					return false;
 				}
 			}
+			if (v) {
+				found = false;
+				for (i1 = i; i1 <= j; i1++) {
+					if (var_in_equation(i1, v)) {
+						found = true;
+						break;
+					}
+				}
+				if (!found) {
+					warning(_("Variable not found."));
+				}
+			}
 			for (i1 = i; i1 <= j; i1++) {
 				if (n_lhs[i1]) {
 					simpv_side(lhs[i1], &n_lhs[i1], v);
-					simpv_side(rhs[i1], &n_rhs[i1], v);
+					if (n_rhs[i1]) {
+						simpv_side(rhs[i1], &n_rhs[i1], v);
+					}
 				}
 			}
 		} while (*cp);
@@ -2613,18 +2852,18 @@ unfactor_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	quick_flag = false, fully_flag = false, power_flag = false;
+	int	quick_flag = false, fraction_flag = false, power_flag = false;
 
 	for (;; cp = skip_param(cp)) {
-		if (strncasecmp(cp, "quick", 5) == 0) {
+		if (strncasecmp(cp, "quick", 4) == 0) {
 			quick_flag = true;
 			continue;
 		}
-		if (strncasecmp(cp, "fully", 4) == 0) {
-			fully_flag = true;
+		if (strncasecmp(cp, "fraction", 4) == 0 || strncasecmp(cp, "fully", 4) == 0) {
+			fraction_flag = true;
 			continue;
 		}
-		if (strncasecmp(cp, "power", 5) == 0) {
+		if (strncasecmp(cp, "power", 4) == 0) {
 			power_flag = true;
 			continue;
 		}
@@ -2633,30 +2872,31 @@ char	*cp;
 	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
 	}
+	partial_flag = !fraction_flag;
 	if (power_flag) {
 		for (; i <= j; i++) {
 			if (n_lhs[i]) {
-				if (fully_flag) {
-					uf_allpower(lhs[i], &n_lhs[i]);
-				} else {
+				if (quick_flag) {
 					uf_power(lhs[i], &n_lhs[i]);
+				} else {
+					uf_allpower(lhs[i], &n_lhs[i]);
 				}
 				elim_loop(lhs[i], &n_lhs[i]);
 				if (n_rhs[i]) {
-					if (fully_flag) {
-						uf_allpower(rhs[i], &n_rhs[i]);
-					} else {
+					if (quick_flag) {
 						uf_power(rhs[i], &n_rhs[i]);
+					} else {
+						uf_allpower(rhs[i], &n_rhs[i]);
 					}
 					elim_loop(rhs[i], &n_rhs[i]);
 				}
 				if (!return_result(i)) {
+					partial_flag = true;
 					return false;
 				}
 			}
 		}
 	} else {
-		partial_flag = !fully_flag;
 		for (; i <= j; i++) {
 			if (n_lhs[i]) {
 				if (quick_flag) {
@@ -2676,8 +2916,8 @@ char	*cp;
 				}
 			}
 		}
-		partial_flag = true;
 	}
+	partial_flag = true;
 	return true;
 }
 
@@ -2736,18 +2976,20 @@ char	*cp;
 	FILE	*fp;
 	char	buf[MAX_CMD_LEN];
 
+	if (security_level >= 3) {
+		error(_("File reading disabled."));
+		return false;
+	}
 	if (*cp == '\0') {
 		error(_("No read file name specified."));
 		return false;
 	}
-	fp = NULL;
 	snprintf(buf, sizeof(buf), "%s.in", cp);
 	fp = fopen(buf, "r");
 	if (fp == NULL) {
 		fp = fopen(cp, "r");
 		if (fp == NULL) {
 			error(_("Can't open input file."));
-			printf(_("Read of file \"%s\" failed.\n"), cp);
 			return false;
 		}
 	} else {
@@ -2758,7 +3000,9 @@ char	*cp;
 	if (rv) {
 		printf(_("Read operation aborted.\n"));
 	} else {
-		printf(_("Finished reading file \"%s\".\n"), cp);
+		if (!quiet_mode) {
+			printf(_("Finished reading file \"%s\".\n"), cp);
+		}
 	}
 #endif
 	fclose(fp);
@@ -2786,19 +3030,7 @@ FILE	*fp;
 		}
 	} else {
 		while ((cp = fgets((char *) tlhs, n_tokens * sizeof(token_type), fp)) != NULL) {
-#if	!SILENT
-			default_color();
-			input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT : PROMPT);
-			if (html_flag)
-				printf("<b>%s</b>", cp);	/* make input bold */
-			else
-				printf("%s", cp);
-#endif
-			if (gfp != stdout) {
-				fprintf(gfp, "%d%s%s", cur_equation + 1, PROMPT, cp);
-			}
-			set_error_level(cp);
-			if (!process(cp)) {
+			if (!display_process(cp)) {
 				longjmp(jmp_save, 3);	/* jump to the above error trap */
 			}
 		}
@@ -2809,6 +3041,38 @@ FILE	*fp;
 #endif
 
 #if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
+static int
+edit_sub(cp)
+char	*cp;
+{
+	char	cl[MAX_CMD_LEN];	/* command line */
+	char	*cp1;
+
+edit_again:
+	cp1 = getenv("EDITOR");
+	if (cp1 == NULL) {
+#if	CYGWIN
+		cp1 = "notepad";
+#else
+		error("EDITOR environment variable not set.");
+		return false;
+#endif
+	}
+	snprintf(cl, sizeof(cl), "%s %s", cp1, cp);
+	if (shell_out(cl)) {
+		error("Error executing editor, check EDITOR environment variable.");
+		printf(_("Command line = \"%s\".\n"), cl);
+		return false;
+	}
+	clear_all();
+	if (!read_cmd(cp)) {
+		if (pause_cmd(_("Prepare to rerun the editor"))) {
+			goto edit_again;
+		}
+	}
+	return true;
+}
+
 /*
  * The edit command.
  */
@@ -2821,6 +3085,10 @@ char	*cp;
 	int	rv;
 	char	tmp_file[PATH_MAX];
 
+	if (security_level) {
+		error(_("Command disabled."));
+		return false;
+	}
 	clean_up();	/* end any redirection */
 	if (*cp == '\0') {
 #if	CYGWIN
@@ -2855,38 +3123,6 @@ char	*cp;
 		return edit_sub(cp);
 	}
 }
-
-static int
-edit_sub(cp)
-char	*cp;
-{
-	char	cl[MAX_CMD_LEN];	/* command line */
-	char	*cp1;
-
-edit_again:
-	cp1 = getenv("EDITOR");
-	if (cp1 == NULL) {
-#if	CYGWIN
-		cp1 = "notepad";
-#else
-		error("EDITOR environment variable not set.");
-		return false;
-#endif
-	}
-	snprintf(cl, sizeof(cl), "%s %s", cp1, cp);
-	if (shell_out(cl)) {
-		error("Error executing editor, check EDITOR environment variable.");
-		printf(_("Command line = \"%s\".\n"), cl);
-		return false;
-	}
-	clear_all();
-	if (!read_cmd(cp)) {
-		if (pause_cmd(_("Prepare to rerun the editor"))) {
-			goto edit_again;
-		}
-	}
-	return true;
-}
 #endif
 
 #if	!SECURE
@@ -2900,6 +3136,10 @@ char	*cp;
 	FILE	*fp;
 	int	rv;
 
+	if (security_level >= 2) {
+		error(_("Command disabled."));
+		return false;
+	}
 	clean_up();	/* end any redirection */
 	if (*cp == '\0') {
 		error(_("No save file name specified."));
