@@ -19,7 +19,6 @@ double	x, y, *radiusp, *thetap;
 	*thetap = atan2(y, x);
 }
 
-#if	!LIBRARY
 /*
  * The roots command.
  */
@@ -29,11 +28,12 @@ char	*cp;
 {
 #define	MAX_ROOT	10000.0	/* root limit needed because more roots become more inaccurate and take longer to calculate */
 
-	complexs	c, c2, check;
-	double		d, k;
-	double		root;
-	double		radius, theta;
-	double		radius_root = 0.0;
+	complexs	c, c2;
+#if	!SILENT
+	complexs	check;
+	double		d;
+#endif
+	double		k, root, radius, theta, radius_root = 0.0;
 	char		buf[MAX_CMD_LEN];
 
 	if (*cp == '\0') {
@@ -54,7 +54,7 @@ char	*cp;
 	}
 	c.re = strtod(cp, &cp);
 	if (*cp && !isspace(*cp)) {
-		printf(_("Invalid number.\n"));
+		error(_("Number expected."));
 		return false;
 	}
 	cp = skip_space(cp);
@@ -65,7 +65,7 @@ char	*cp;
 	}
 	c.im = strtod(cp, &cp);
 	if (*cp) {
-		printf(_("Invalid number.\n"));
+		error(_("Number expected."));
 		return false;
 	}
 	if (c.re == 0.0 && c.im == 0.0) {
@@ -91,31 +91,38 @@ char	*cp;
 			c2.re = radius_root * cos((theta + 2.0 * k * M_PI) / root);
 			c2.im = radius_root * sin((theta + 2.0 * k * M_PI) / root);
 			complex_fixup(&c2);
-			if (c2.im == 0.0) {
-				fprintf(gfp, "%.12g\n", c2.re);
-			} else {
-				fprintf(gfp, "%.12g %+.12g*i\n", c2.re, c2.im);
+			if (c2.re || c2.im == 0.0) {
+				fprintf(gfp, "%.12g ", c2.re);
 			}
+			if (c2.im) {
+				fprintf(gfp, "%+.12g*i", c2.im);
+			}
+			fprintf(gfp, "\n");
+#if	!SILENT
 			check = c2;
 			for (d = 1.0; d < root; d += 1.0) {
 				check = complex_mult(check, c2);
 			}
 			complex_fixup(&check);
-			if (check.im == 0.0) {
-				printf(_("Inverse check: %.12g\n\n"), check.re);
-			} else {
-				printf(_("Inverse check: %.12g %+.12g*i\n\n"), check.re, check.im);
+			printf(_("Inverse check:"));
+			if (check.re || check.im == 0.0) {
+				printf(" %.12g", check.re);
 			}
+			if (check.im) {
+				printf(" %+.12g*i", check.im);
+			}
+			printf("\n\n");
+#endif
 		}
 	}
 	return true;
 }
-#endif
 
 /*
  * Approximate roots of complex numbers in an equation side:
- * (complex^real) and (real^complex) and (complex^complex).
+ * (complex^real) and (real^complex) and (complex^complex) all result in a complex number.
  * This only gives one root, even when there may be many roots.
+ * Works best when the equation side has been approximated before this.
  *
  * Return true and display a warning message if the equation side was modified.
  */
@@ -188,6 +195,19 @@ start_over:
 	return modified;
 }
 
+/*
+ * Approximate all roots of complex numbers in an equation side.
+ */
+void
+approximate_complex_roots(equation, np)
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to length of equation side */
+{
+	do {
+		elim_loop(equation, np);
+	} while (complex_root_simp(equation, np));
+}
+
 /*
  * Get a constant, if the passed expression evaluates to a constant.
  * This should not be called from low level routines.
@@ -253,6 +273,8 @@ double		*dp;	/* pointer to returned double */
 
 /*
  * Get the value of a constant complex number expression.
+ * Doesn't always work unless expression is approximated first
+ * with something like the approximate command.
  *
  * If successful return true with complex number in *cp.
  */
