@@ -84,7 +84,7 @@ long		v;
 				break;
 			default:
 /* Oops.  More than one operator on the same level in this expression. */
-				error("Internal error in d_recurse(): differentiating with unparenthesized operators is not allowed.");
+				error_bug("Internal error in d_recurse(): differentiating with unparenthesized operators is not allowed.");
 				return false;
 			}
 			op = equation[endloc].token.operatr;
@@ -305,6 +305,9 @@ char	*cp;
 	}
 	i = next_espace();
 	if (n_rhs[cur_equation]) {
+		if (!solved_equation(cur_equation)) {
+			warning(_("Not a solved equation.  Only the RHS will be differentiated."));
+		}
 		source = rhs[cur_equation];
 		nps = &n_rhs[cur_equation];
 		dest = rhs[i];
@@ -491,12 +494,16 @@ char	*cp;
 	int		our_nlhs, our_nrhs;
 	token_type	*ep, *source, *dest;
 	int		n1, *nps, *np;
-	int		flag = false;
+	int		simplify_flag = true;
 
 	cp_start = cp;
 	if (current_not_defined()) {
 		return false;
 	}
+	if (strcmp_tospace(cp, "nosimplify") == 0) {
+		simplify_flag = false;
+		cp = skip_param(cp);
+	}
 	i = next_espace();
 	blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 	n_lhs[i] = n_lhs[cur_equation];
@@ -579,10 +586,19 @@ char	*cp;
 				return false;
 			}
 		} else {
-			warning(_("Derivatives will be taken until they reach zero..."));
 			order = LONG_MAX - 1L;
+#if	!SILENT
+			printf(_("Derivatives will be taken until they reach zero...\n"));
+#endif
 		}
 	}
+#if	!SILENT
+	if (simplify_flag) {
+		printf(_("Computing and simplifying...\n"));
+	} else {
+		printf(_("Computing...\n"));
+	}
+#endif
 	n = 0;
 	i1 = 0;
 	blt(dest, source, *nps * sizeof(token_type));
@@ -646,12 +662,14 @@ loop_again:
 	n1 += 4;
 	for (; i1 < n1; i1++)
 		dest[i1].level++;
-	uf_simp(dest, &n1);
-	if (!flag && exp_contains_infinity(dest, n1)) {
-		warning(_("Result invalid because it contains infinity or NaN."));
-		flag = true;
+	if (simplify_flag) {
+		uf_simp(dest, &n1);
 	}
 	side_debug(1, dest, n1);
+	if (exp_contains_infinity(dest, n1)) {
+		error(_("Result invalid because it contains infinity or NaN."));
+		return false;
+	}
 	if (n < order) {
 		if (n > 0) {
 			if (!differentiate(rhs[our], &our_nrhs, v)) {
@@ -659,7 +677,7 @@ loop_again:
 				return false;
 			}
 		}
-		symb_flag = true;
+		symb_flag = symblify;
 		simpa_side(rhs[our], &our_nrhs, true, true);
 		symb_flag = false;
 		if (our_nrhs != 1 || rhs[our][0].kind != CONSTANT || rhs[our][0].token.constant != 0.0) {
@@ -740,6 +758,9 @@ char	*cp;
 		error(_("Variable not found."));
 		return false;
 	}
+	if (*cp == '=') {
+		cp = skip_space(cp + 1);
+	}
 	if (*cp) {
 		input_column += (cp - cp_start);
 		if ((cp = parse_expr(tes, &n_tes, cp)) == NULL || n_tes <= 0) {
@@ -752,12 +773,12 @@ char	*cp;
 			return false;
 		}
 	}
-/* copy the current equation to a new equation space and work on the copy: */
+/* copy the current equation to a new equation space, then simplify and work on the copy: */
 	copy_espace(cur_equation, i);
+	simpa_side(rhs[i], &n_rhs[i], false, false);
+
 /* see if the limit expression is positive infinity: */
 	simp_loop(tes, &n_tes);
-
-	debug_string(0, "Solving...");
 	if (n_tes == 1 && tes[0].kind == CONSTANT && tes[0].token.constant == INFINITY) {
 /* To take the limit to positive infinity, */
 /* replace infinity with zero and replace the limit variable with its reciprocal: */
@@ -773,7 +794,9 @@ char	*cp;
 		n_tlhs = 3;
 		subst_var_with_exp(rhs[i], &n_rhs[i], tlhs, n_tlhs, v);
 	}
+
 /* General limit taking, solve for the limit variable: */
+	debug_string(0, "Solving...");
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
@@ -784,8 +807,8 @@ char	*cp;
 /* replace the limit variable (LHS) with the limit expression: */
 	blt(lhs[i], tes, n_tes * sizeof(token_type));
 	n_lhs[i] = n_tes;
-/* symbolically simplify the RHS: */
-	symb_flag = true;
+/* simplify the RHS: */
+	symb_flag = symblify;
 	simpa_side(rhs[i], &n_rhs[i], false, false);
 	symb_flag = false;
 	if (exp_contains_nan(rhs[i], n_rhs[i])) {
@@ -797,10 +820,8 @@ char	*cp;
 		error(_("Can't take the limit because solve failed."));
 		return false;
 	}
-/* symbolically simplify before returning the result: */
-	symb_flag = true;
+/* simplify before returning the result: */
 	simpa_side(rhs[i], &n_rhs[i], false, false);
-	symb_flag = false;
 	if (exp_contains_nan(rhs[i], n_rhs[i])) {
 		error(_("Unable to take limit; result contains NaN (Not a Number)."));
 		return false;
