@@ -21,20 +21,20 @@ static double skip_multiples[] = {	/* Additive array that skips over multiples o
 };	/* sum of all numbers = 210 = (2*3*5*7) */
 
 /*
- * Factor the integer in "start".
+ * Factor the integer in "value".
  * Store the prime factors in the unique[] array.
  *
  * Return true if successful.
  */
 int
-factor_one(start)
-double	start;
+factor_one(value)
+double	value;
 {
 	int	i;
 	double	d;
 
 	uno = 0;
-	nn = start;
+	nn = value;
 	if (nn == 0.0) {
 		return false;
 	}
@@ -61,9 +61,8 @@ double	start;
 	if (nn != 1.0) {
 		try_factor(nn);
 	}
-	if (start != multiply_out_unique()) {
-		error("Internal error factoring integers.");
-		return false;
+	if (value != multiply_out_unique()) {
+		error_bug("Internal error factoring integers.");
 	}
 	return true;
 }
@@ -92,8 +91,9 @@ double	arg;
 }
 
 /*
- * Convert unique[] back into an integer.
- * Return the double integer.
+ * Convert unique[] back into the single integer it represents,
+ * which was the value passed in the last call to factor_one(value).
+ * Nothing is changed and the value is returned.
  */
 double
 multiply_out_unique()
@@ -111,14 +111,18 @@ multiply_out_unique()
 }
 
 /*
- * Display the prime factors in the unique[] array.
+ * Display the integer prime factors in the unique[] array.
+ * Must have had a successful call to factor_one(value) previously,
+ * to fill out unique[] with the prime factors of value.
  */
 void
 display_unique()
 {
 	int	i;
+	double	value;
 
-	fprintf(gfp, "%.0f = ", multiply_out_unique());
+	value = multiply_out_unique();
+	fprintf(gfp, "%.0f = ", value);
 	for (i = 0; i < uno;) {
 		fprintf(gfp, "%.0f", unique[i]);
 		if (ucnt[i] > 1) {
@@ -140,6 +144,11 @@ display_unique()
 int
 is_prime()
 {
+	double	value;
+
+	value = multiply_out_unique();
+	if (value < 2.0)
+		return false;
 	if (uno == 1 && ucnt[0] == 1)
 		return true;
 	return false;
@@ -267,6 +276,8 @@ int		factor_flag;
  *
  * If "level_code" is 3, nothing is done.
  *
+ * Add 4 to "level_code" to always factor out the GCD of rational coefficients.
+ *
  * Return true if equation side was modified.
  */
 int
@@ -275,7 +286,7 @@ token_type	*equation;	/* pointer to the beginning of equation side */
 int		*np;		/* pointer to length of equation side */
 int		level_code;	/* see above */
 {
-	if (level_code > 2)
+	if (level_code == 3)
 		return false;
 	return fc_recurse(equation, np, 0, 1, level_code);
 }
@@ -289,7 +300,7 @@ int		level_code;
 	int	i, j, k;
 	int	op;
 	double	d, minimum = 1.0, cogcd = 1.0;
-	int	improve_readability, first = true, neg_flag = true, modified = false;
+	int	improve_readability, gcd_flag, first = true, neg_flag = true, modified = false;
 	int	op_count = 0, const_count = 0;
 
 	for (i = loc; i < *np && equation[i].level >= level;) {
@@ -304,7 +315,8 @@ int		level_code;
 	}
 	if (modified)
 		return true;
-	improve_readability = (level_code > 1 || (level_code && (level == 1)));
+	improve_readability = ((level_code & 3) > 1 || ((level_code & 3) && (level == 1)));
+	gcd_flag = ((improve_readability && integer_coefficients) || (level_code & 4));
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level == level) {
 			switch (equation[i].kind) {
@@ -338,7 +350,7 @@ int		level_code;
 			} else {
 				if (minimum > d)
 					minimum = d;
-				if (integer_coefficients && improve_readability && cogcd)
+				if (gcd_flag && cogcd != 0.0)
 					cogcd = gcd_verified(d, cogcd);
 			}
 		} else {
@@ -366,7 +378,7 @@ int		level_code;
 						} else {
 							if (minimum > d)
 								minimum = d;
-							if (integer_coefficients && improve_readability && cogcd)
+							if (gcd_flag && cogcd != 0.0)
 								cogcd = gcd_verified(d, cogcd);
 						}
 						i = j;
@@ -384,7 +396,7 @@ int		level_code;
 			} else {
 				if (minimum > 1.0)
 					minimum = 1.0;
-				if (integer_coefficients && improve_readability && cogcd)
+				if (gcd_flag && cogcd != 0.0)
 					cogcd = gcd_verified(1.0, cogcd);
 			}
 			i = j;
@@ -392,7 +404,7 @@ int		level_code;
 		}
 		i++;
 	}
-	if (integer_coefficients && improve_readability && cogcd && fmod(cogcd, 1.0) == 0.0) {
+	if (gcd_flag && cogcd != 0.0 /* && fmod(cogcd, 1.0) == 0.0 */) {
 		minimum = cogcd;
 	}
 	if (first || op_count == 0 || const_count > 1 || (!neg_flag && minimum == 1.0))
@@ -411,6 +423,12 @@ int		level_code;
 					d = equation[i].token.constant;
 				}
 			}
+#if	true
+			if (!gcd_flag && minimum >= 1.0) {
+				minimum = 1.0;
+				break;
+			}
+#endif
 			if ((minimum < 1.0 && fmod(d, 1.0) == 0.0) || (fmod(d, minimum) != 0.0)) {
 				minimum = 1.0;
 				break;
