@@ -10,9 +10,9 @@
  * Return the Greatest Common Divisor (GCD) of doubles d1 and d2,
  * by using the Euclidean GCD algorithm.
  *
- * The GCD is defined as the largest positive number that evenly divides both "d1" and "d2".
+ * The GCD is defined as the largest positive number that evenly divides both d1 and d2.
+ * This should always works perfectly and exactly with two integers up to MAX_K_INTEGER.
  * Will usually work with non-integers, but there may be some floating point error.
- * Always works perfectly with integers up to MAX_K_INTEGER.
  *
  * Returns 0 on failure, otherwise returns the positive GCD.
  */
@@ -61,17 +61,22 @@ double	d1, d2;
 /*
  * Return the verified exact Greatest Common Divisor (GCD) of doubles d1 and d2.
  *
- * Returns 0 on failure, otherwise returns the verified positive GCD.
+ * Returns 0 on failure or inexactness, otherwise returns the verified positive GCD result.
+ * Result is not necessarily integer unless both d1 and d2 are integer.
  */
 double
 gcd_verified(d1, d2)
 double	d1, d2;
 {
-	double	divisor;
+	double	divisor, d3, d4;
 
 	divisor = gcd(d1, d2);
 	if (divisor != 0.0) {
-		if (gcd(d1 / divisor, d2 / divisor) != 1.0)
+		d3 = d1 / divisor;
+		d4 = d2 / divisor;
+		if (fmod(d3, 1.0) != 0.0 || fmod(d4, 1.0) != 0.0)
+			return 0.0;
+		if (gcd(d3, d4) != 1.0)
 			return 0.0;
 	}
 	return divisor;
@@ -158,6 +163,29 @@ double	*denominatorp;	/* returned denominator */
 	return false;
 }
 
+/*
+ * Display the specified value.
+ * If it is equal to a simple fraction, display that too.
+ *
+ * Return true if a fraction was displayed.
+ */
+int
+display_fraction(value)
+double	value;
+{
+	double	d4, d5;
+	int	rv = false;
+
+	f_to_fraction(value, &d4, &d5);
+	fprintf(gfp, "%.*g", precision, value);
+	if (d5 != 1.0) {
+		fprintf(gfp, " = %.*g/%.*g", precision, d4, precision, d5);
+		rv = true;
+	}
+	fprintf(gfp, "\n");
+	return rv;
+}
+
 /*
  * Convert non-integer constants in an equation side to fractions,
  * when exactly equal to simple fractions.
