@@ -40,14 +40,16 @@ int		case_sensitive_flag = true;		/* "set case_sensitive" flag */
 int		factor_int_flag;			/* factor integers when displaying expressions */
 int		display2d = true;			/* "set display2d" flag */
 int		preserve_surds = true;			/* set option to preserve roots like (2^.5) */
+int		rationalize_denominators = true;	/* try to rationalize denominators if true */
 int		modulus_mode = 2;				/* true for mathematically correct modulus */
 volatile int	screen_columns = STANDARD_SCREEN_COLUMNS;	/* screen width of the terminal; 0 = infinite */
 volatile int	screen_rows = STANDARD_SCREEN_ROWS;		/* screen height of the terminal; 0 = infinite */
 int		finance_option;				/* for displaying dollars and cents */
-int		autosolve = true;			/* Allows solving by typing the variable name at the prompt */
+int		autosolve = true;			/* Allows solving by typing the variable name at the main prompt */
 int		autocalc = true;			/* Allows automatically calculating a numerical expression */
+int		autoselect = true;			/* Allows selecting equation spaces by typing the number */
 char		special_variable_characters[256] = "\\"; /* user defined characters for variable names, 0 terminated */
-int		integer_coefficients = true;		/* if true, factor out the GCD of integer coefficients */
+int		integer_coefficients = false;		/* if true, factor out the GCD of rational coefficients */
 int		right_associative_power;		/* if true, evaluate power operators right to left */
 int		negate_highest_precedence = true;	/* if true, negation (-x) has the highest precedence */
 int		power_starstar;				/* if true, display power operator as "**", otherwise "^" */
@@ -55,8 +57,12 @@ int		power_starstar;				/* if true, display power operator as "**", otherwise "^
 int		debug_level;				/* current debug level */
 #endif
 
-/* variables having to do with color mode */
-int		color_flag = true;		/* "set color" flag, true for color mode */
+/* variables having to do with color output mode */
+#if	LIBRARY || NO_COLOR
+int		color_flag = false;		/* library shouldn't default to color mode */
+#else
+int		color_flag = true;		/* "set color" flag, true for color output mode */
+#endif
 int		bold_colors;			/* "set bold_colors" flag */
 int		cur_color = -1;			/* current color on the terminal */
 int		html_flag;			/* true for HTML output mode */
@@ -70,6 +76,10 @@ char		*prog_name = "mathomatic";	/* name of this program */
 char		*var_names[MAX_VAR_NAMES];	/* index for storage of variable name strings */
 char		var_str[MAX_VAR_LEN+80];	/* temp storage for listing a variable name */
 char		prompt_str[MAX_PROMPT_LEN];	/* temp storage for the prompt string */
+#if	!SECURE && !LIBRARY
+char		rc_file[PATH_MAX];		/* pathname for the set options startup file */
+#endif
+
 #if	CYGWIN
 char		*dir_path;			/* directory path to the executable, only set and used in CYGWIN version */
 #endif
@@ -92,8 +102,9 @@ int		test_mode;		/* test mode flag (-t) */
 int		quiet_mode;		/* quiet mode (-q, don't display prompts) */
 int		echo_input;		/* if true, echo input and don't use readline */
 int		readline_enabled = true;	/* set to false (-r) to disable readline */
-int		partial_flag;		/* normally true for partial unfactoring, false for "unfactor fully" */
-int		symb_flag;		/* true during "simplify symbolic" */
+int		partial_flag;		/* normally true for partial unfactoring, false for "unfactor fraction" */
+int		symb_flag;		/* true during "simplify symbolic", which is not 100% mathematically correct */
+int		symblify = true;	/* if true, set symb_flag when helpful during solving, etc. */
 int		high_prec;		/* flag to output constants in higher precision (used when saving equations) */
 int		input_column;		/* current column number on the screen at the beginning of a parse */
 int		sign_cmp_flag;		/* true when all "sign" variables are to compare equal */
@@ -101,8 +112,11 @@ int		domain_check;		/* flag to track domain errors in the pow() function */
 int		approximate_roots;	/* true if in calculate command (force approximation of roots like (2^.5)) */
 volatile int	abort_flag;		/* if true, abort current operation */
 int		pull_number;		/* equation space number to pull when using the library */
+#if	!SECURE
+int		security_level;		/* current enforced security level for session  */
+#endif
 
 /* library variables go here */
-const char	*error_str;		/* last error string */
 char		*result_str;		/* returned result when using as library */
+const char	*error_str;		/* last error string */
 const char	*warning_str;		/* last warning string */
