@@ -7,6 +7,7 @@
  */
 
 #include "includes.h"
+#include "license.h"	/* the current license for Mathomatic */
 
 /*
  * The following structure is used for each Mathomatic command.
@@ -30,39 +31,36 @@ static com_type com_list[] = {
 #if	!LIBRARY
 {	"calculate",	NULL,			calculate_cmd,		"[\"repeat\"] [\"factor\"] [variable number-of-iterations]",	"Temporarily plug in values for variables and approximate." },
 #endif
-{	"clear",	NULL,			clear_cmd,		"[equation-number-ranges]",				"Delete expressions stored in memory so equation spaces can be reused.", "Tip: Use \"clear all\" to restart Mathomatic." },
-#if	!LIBRARY
-{	"code",		NULL,			code_cmd,		"[\"c\" or \"java\" or \"python\" or \"integer\"] [equation-number-ranges]",	"Output C, Java, or Python code for the specified equations.", "Related commands: simplify, optimize, and variables." },
-#endif
-{	"compare",	NULL,			compare_cmd,		"[\"symbolic\"] equation-number [\"with\" equation-number]",	"Compare two equation spaces to see if they are the same." },
-{	"copy",		NULL,			copy_cmd,		"[equation-number-range]",				"Duplicate the specified equation spaces." },
+{	"clear",	NULL,			clear_cmd,		"[equation-number-ranges]",				"Delete expressions stored in memory so equation spaces can be reused.", "Tip: Use \"clear all\" to quickly restart Mathomatic." },
+{	"code",		NULL,			code_cmd,		"[\"c\" or \"java\" or \"python\" or \"integer\"] [equation-number-ranges]",	"Output C, Java, or Python code for the specified equations.", "Related commands: simplify, optimize, and variables" },
+{	"compare",	NULL,			compare_cmd,		"[\"symbolic\"] equation-number [\"with\" equation-number]",	"Compare two equation spaces to see if they are mathematically the same." },
+{	"copy",		NULL,			copy_cmd,		"[equation-number-range]",				"Duplicate the contents of the specified equation spaces." },
 {	"derivative",	"differentiate",	derivative_cmd,		"[\"nosimplify\"] [variable or \"all\"] [order]",	"Symbolically differentiate and simplify, order times." },
 {	"display",	NULL,			display_cmd,		"[\"factor\"] [equation-number-ranges]",			"Display equation spaces in pretty multi-line (2D) fraction format." },
-#if	!LIBRARY
 {	"divide",	NULL,			divide_cmd,		"[variable]",						"Prompt for 2 numbers or polynomials and divide. Give result and GCD." },
-#endif
 {	"echo",		NULL,			echo_cmd,		"[text]",						"Output a line of text, followed by a newline." },
-#if	!LIBRARY && (UNIX || CYGWIN) && !SECURE
+#if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
 {	"edit",		NULL,			edit_cmd,		"[file-name]",						"Edit all equation spaces or an input file, then read them in." },
 #endif
 {	"eliminate",	NULL,			eliminate_cmd,		"[\"repeat\"] variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations." },
-{	"extrema",	NULL,			extrema_cmd,		"[variable] [order]",					"Find possible local minimums and maximums of the current expression." },
-{	"factor",	NULL,			factor_cmd,		"[\"numbers\" [integers]] or [equation-number-range] [variables]",	"Factor variables in equation spaces or factor given integers." },
-{	"fraction",	NULL,			fraction_cmd,		"[equation-number-range]",				"Convert to a single simple fraction, like \"simplify fraction\" does." },
+{	"extrema",	NULL,			extrema_cmd,		"[variable] [order]",					"Show where the slope equals zero." },
+{	"factor",	"collect",		factor_cmd,		"[\"number\" [integers]] or [\"power\"] [equation-number-range] [variables]",	"Factor variables in equation spaces or factor given integers." },
+{	"fraction",	"together",		fraction_cmd,		"[equation-number-range]",				"Convert expression to a single simple fraction, if any fractions are in it." },
 {	"help",		NULL,			help_cmd,		"[topic or command-names]",				"Short, built-in help and reference." },
-{	"imaginary",	NULL,			imaginary_cmd,		"[variable]",						"Copy the imaginary part of the current expression.", "Related command: real." },
+{	"imaginary",	NULL,			imaginary_cmd,		"[variable]",						"Copy the imaginary part of the current expression.", "Related command: real" },
 {	"integrate",	"integral",		integrate_cmd,		"[\"constant\" or \"definite\"] variable [order]",	"Symbolically integrate polynomials order times, then simplify." },
 {	"laplace",	NULL,			laplace_cmd,		"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials." },
 {	"limit",	NULL,			limit_cmd,		"variable expression",					"Take the limit as variable goes to expression (experimental)." },
-{	"list",		NULL,			list_cmd,		"[\"export\" or \"maxima\"] [equation-number-ranges]",	"Display equation spaces in single-line format." },
+{	"list",		NULL,			list_cmd,		"[\"export\" or \"maxima\" or \"gnuplot\"] [equation-number-ranges]",	"Display equation spaces in single-line format." },
 {	"nintegrate",	NULL,			nintegrate_cmd,		"[\"trapezoid\"] variable [partitions]",		"Do numerical definite integration using Simpson's rule."},
-#if	!LIBRARY
-{	"optimize",	NULL,			optimize_cmd,		"[equation-number-range]",				"Split up equations into smaller, more efficient equations.", "Related command: code." },
-#endif
+{	"optimize",	NULL,			optimize_cmd,		"[equation-number-range]",				"Split up equations into smaller, more efficient equations.", "Related command: code" },
 #if	!LIBRARY
 {	"pause",	NULL,			pause_cmd,		"[text]",						"Wait for user to press the Enter key. Optionally display a message." },
 #endif
-{	"product",	NULL,			product_cmd,		"variable start end [step-size]",			"Compute the product as variable goes from start to end.", "Related command: sum." },
+#if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
+{	"plot",		NULL,			plot_cmd,		"[expression]",						"Automatically plot expression in 2D or 3D with Gnuplot." },
+#endif
+{	"product",	NULL,			product_cmd,		"variable start end [step-size]",			"Compute the product as variable goes from start to end.", "Related command: sum" },
 #if	READLINE
 {	"push",		NULL,			push_cmd,		"[equation-number-range]",				"Push the specified expressions into the readline history." },
 #endif
@@ -72,29 +70,46 @@ static com_type com_list[] = {
 #if	!SECURE
 {	"read",		NULL,			read_cmd,		"file-name",						"Read in a text file as if it was typed in." },
 #endif
-{	"real",		NULL,			real_cmd,		"[variable]",						"Copy the real part of the current expression.", "Related command: imaginary." },
+{	"real",		NULL,			real_cmd,		"[variable]",						"Copy the real part of the current expression.", "Related command: imaginary" },
 {	"replace",	NULL,			replace_cmd,		"[variables [\"with\" expression]]",			"Substitute variables in the current equation with expressions." },
-#if	!LIBRARY
 {	"roots",	NULL,			roots_cmd,		"root real-part imaginary-part",			"Display all the roots of a complex number." },
-#endif
 #if	!SECURE
-{	"save",		NULL,			save_cmd,		"file-name",						"Save all equation spaces in a text file.", "Related command: read." },
+{	"save",		NULL,			save_cmd,		"file-name",						"Save all equation spaces in a text file.", "Related command: read" },
 #endif
-{	"set",		NULL,			set_cmd,		"[[\"no\"] option]",					"Set or display various session options." },
-{	"simplify",	NULL,			simplify_cmd,		"[\"sign\"] [\"symbolic\"] [\"quick\"] [\"fraction\"] [equation-number-range]",	"Completely simplify expressions." },
-{	"solve",	NULL,			solve_cmd,		"[\"verify\"] variable or \"0\"",			"Solve the current equation for a variable or for zero." },
-{	"sum",		NULL,			sum_cmd,		"variable start end [step-size]",			"Compute the summation as variable goes from start to end.", "Related command: product." },
+{	"set",		NULL,			set_cmd,		"[[\"no\"] option] ...",				"Display, set, or save current session options.", "Tip: Type \"set\" by itself to show all current option settings." },
+{	"simplify",	NULL,			simplify_cmd,		"[\"sign\"] [\"symbolic\"] [\"quick\"] [\"quickest\"] [\"fraction\"] [equation-number-range]",	"Completely simplify expressions." },
+{	"solve",	NULL,			solve_cmd,		"[\"verify\"] [\"for\"] variable or \"0\"",		"Solve the current equation for a variable or for zero." },
+{	"sum",		NULL,			sum_cmd,		"variable start end [step-size]",			"Compute the summation as variable goes from start to end.", "Related command: product" },
 #if	!LIBRARY
 {	"tally",	NULL,			tally_cmd,		"[\"average\"]",					"Prompt for and add entries, show total and optionally the average." },
 #endif
-{	"taylor",	NULL,			taylor_cmd,		"variable order point",					"Compute the Taylor series expansion of the current expression." },
-{	"unfactor",	"expand",		unfactor_cmd,		"[\"fully\"] [\"quick\"] [\"power\"] [equation-number-range]",	"Algebraically expand (multiply out) expressions." },
-#if	!LIBRARY
-{	"variables",	NULL,			variables_cmd,		"[\"c\" or \"java\" or \"integer\"] [equation-number-range]",	"Output all variable names within the specified equations.", "Related command: code." },
-#endif
-{	"version",	NULL,			version_cmd,		"",							"Display version number, compile flags, and memory usage." }
+{	"taylor",	NULL,			taylor_cmd,		"[\"nosimplify\"] variable order point",		"Compute the Taylor series expansion of the current expression." },
+{	"unfactor",	"expand",		unfactor_cmd,		"[\"fraction\"] [\"quick\"] [\"power\"] [equation-number-range]",	"Algebraically expand (multiply out) expressions." },
+{	"variables",	NULL,			variables_cmd,		"[\"c\" or \"java\" or \"integer\"] [equation-number-range]",	"Show all variable names used within the specified expressions.", "Related command: code" },
+{	"version",	NULL,			version_cmd,		"",							"Display Mathomatic version and license information." }
 };
 
+#if	!LIBRARY
+char	*example_strings[] = {
+	"; The derivative of the absolute value function:\n",
+	"|x|\n",
+	"derivative\n",
+	"pause\n",
+	"\n",
+	"; Solve and simplify an equation:\n",
+	"fahrenheit = (9*celsius/5) + 32\n",
+	"solve for celsius\n",
+	"simplify\n",
+	"variables ; show all variables that occur in an expression\n",
+	"pause\n",
+	"\n",
+	"; Expand into a polynomial:\n",
+	"(x+1)^5\n",
+	"unfactor\n",
+	NULL
+};
+#endif
+
 /*
  * Process mathematical expression input in Mathomatic,
  * with no solving and no automatic calculation.
@@ -127,10 +142,12 @@ char	*cp;
 
 /*
  * Process equation and expression input in Mathomatic,
- * either swapping, selecting, storing, solving (autosolve), or calculating (autocalc).
+ * either swapping equation sides, selecting an equation space (autoselect),
+ * solving the current equation (autosolve), calculating a numerical expression (autocalc),
+ * or storing.
  *
- * Parse the mathematical expression text in "cp" and solve the current equation for it
- * or place it in equation space "n" if it is not a solve variable.
+ * Parse the mathematical expression text in "cp" and perform one of the above operations,
+ * or store it in equation space "n" and display.
  *
  * Return true if successful.
  */
@@ -152,7 +169,7 @@ char	*cp;
 		}
 		if (n_lhs[n] == 0 && n_rhs[n] == 0) {
 			if (strcmp(cp, "=") == 0 && n_lhs[cur_equation] > 0 && n_rhs[cur_equation] > 0) {
-				debug_string(0, _("Swapping sides of the current equation..."));
+				debug_string(0, _("Swapping both sides of the current equation..."));
 				n = cur_equation;
 				i = n_lhs[n];
 				blt(tes, lhs[n], n_lhs[n] * sizeof(token_type));
@@ -182,7 +199,7 @@ char	*cp;
 				lhs[n][0] = zero_token;
 				goto return_ok;
 			}
-			if (n_lhs[n] == 1 && lhs[n][0].kind == CONSTANT && fmod(lhs[n][0].token.constant, 1.0) == 0.0
+			if (autoselect && n_lhs[n] == 1 && lhs[n][0].kind == CONSTANT && fmod(lhs[n][0].token.constant, 1.0) == 0.0
 			    && lhs[n][0].token.constant > 0.0 && lhs[n][0].token.constant <= n_equations) {
 				/* easy selecting of equation spaces by just typing in the equation number */
 				cur_equation = lhs[n][0].token.constant - 1;
@@ -228,8 +245,8 @@ return_ok:
 }
 
 /*
- * Process a line of input to Mathomatic.
- * It may be a command, an expression, an equation, etc.
+ * Run a line of main prompt input to Mathomatic.
+ * It may be a command, an expression or equation to store, etc.
  *
  * Return true if successful.
  */
@@ -239,13 +256,16 @@ char	*cp;
 {
 	char	*cp1;
 	char	*cp_start;
-	int	i, i1;
+	int	i;
 	int	len;
 	int	rv;
 	char	buf2[MAX_CMD_LEN];
-	char	*filename;
+#if	!SECURE
 	FILE	*fp;
+	char	*filename;
 	int	append_flag;
+	int	i1;
+#endif
 
 	init_gvars();		/* make sure we are in the default state */
 	set_sign_array();	/* register all sign variables so that the next ones will be unique */
@@ -288,13 +308,13 @@ char	*cp;
 	}
 #if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
 /* handle shell escape */
-	if (*cp == '!') {
+	if (security_level == 0 && *cp == '!') {
 		cp1 = getenv("SHELL");
 		if (cp1 == NULL) {
 			cp1 = "/bin/sh";
 		}
 		if (access(cp1, X_OK)) {
-			error("Shell not found or not executable, check SHELL environment variable.");
+			error(_("Shell not found or not executable, check SHELL environment variable."));
 			return false;
 		}
 		cp = skip_space(cp + 1);
@@ -319,45 +339,48 @@ char	*cp;
 		    || (com_list[i].secondary_name && len >= min(4, strlen(com_list[i].secondary_name)) && strncasecmp(cp, com_list[i].secondary_name, len) == 0)) {
 			cp1 = skip_space(cp1);
 			input_column += (cp1 - cp_start);
+/* Copy the command line to buf2 and use it, because the original string may be overwritten. */
 			if (my_strlcpy(buf2, cp1, sizeof(buf2)) >= sizeof(buf2)) {
 				error(_("Command line too long."));
 				return false;
 			}
-			fp = NULL;
 #if	!SECURE
-			/* handle output redirection */
-			append_flag = false;
-			filename = NULL;
-			for (i1 = strlen(buf2) - 1; i1 >= 0; i1--) {
-				if (buf2[i1] == '>') {
-					filename = skip_space(&buf2[i1+1]);
-					if (i1 && buf2[i1-1] == '>') {
-						i1--;
-						append_flag = true;
+			fp = NULL;
+			if (security_level < 2) {
+				/* handle output redirection */
+				append_flag = false;
+				filename = NULL;
+				for (i1 = strlen(buf2) - 1; i1 >= 0; i1--) {
+					if (buf2[i1] == '>') {
+						filename = skip_space(&buf2[i1+1]);
+						if (i1 && buf2[i1-1] == '>') {
+							i1--;
+							append_flag = true;
+						}
+						buf2[i1] = '\0';
+						break;
 					}
-					buf2[i1] = '\0';
-					break;
 				}
-			}
-			if (filename) {
-				if (append_flag) {
-					fp = fopen(filename, "a");
-				} else {
-					fp = fopen(filename, "w");
+				if (filename) {
+					if (append_flag) {
+						fp = fopen(filename, "a");
+					} else {
+						fp = fopen(filename, "w");
+					}
+					if (fp == NULL) {
+						error(_("Can't open redirected output file for writing."));
+						return false;
+					}
+					if (gfp != stdout) {
+						fclose(gfp);	/* make sure previous redirection file is closed */
+					}
+					gfp = fp;
 				}
-				if (fp == NULL) {
-					error(_("Can't open redirected output file for writing."));
-					return false;
-				}
-				if (gfp != stdout) {
-					fclose(gfp);	/* make sure previous redirection file is closed */
-				}
-				gfp = fp;
 			}
 #endif
 			remove_trailing_spaces(buf2);
+			pull_number = 1;
 /* execute the command by calling the command function */
-			pull_number = 0;
 			rv = (*com_list[i].func)(buf2);
 #if	!SECURE
 			if (fp && gfp != stdout) {
@@ -365,8 +388,7 @@ char	*cp;
 				gfp = stdout;
 			}
 #endif
-#if	false
-/* This shouldn't display usage while reading in files; only display when the user has typed the command. */
+#if	!SILENT
 			if (!rv && debug_level >= 0) {
 				printf("Command usage: %s %s\n", com_list[i].name, com_list[i].usage);
 			}
@@ -380,6 +402,34 @@ char	*cp;
 	return process_parse(i, cp);
 }
 
+/*
+ * Display and process Mathomatic main prompt input that ends with a newline.
+ *
+ * Return true if successful.
+ */
+int
+display_process(cp)
+char	*cp;	/* string to process, will be modified */
+{
+	if (cp == NULL)
+		return false;
+#if	LIBRARY
+	fprintf(gfp, "%d%s%s", cur_equation + 1, PROMPT_STR, cp);
+#else
+	default_color();
+	input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT_STR : PROMPT_STR);
+	if (html_flag)
+		printf("<b>%s</b>", cp);	/* make input bold */
+	else
+		printf("%s", cp);
+	if (gfp != stdout) {
+		fprintf(gfp, "%d%s%s", cur_equation + 1, PROMPT_STR, cp);
+	}
+#endif
+	set_error_level(cp);
+	return process(cp);
+}
+
 #if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
 /*
  * Execute a system command.  Note that system(3) requires "/bin/sh".
@@ -392,7 +442,11 @@ char	*cp;	/* system command string */
 {
 	int	rv;
 
+	if (security_level) {
+		error_bug("Shelling out disabled.");
+	}
 	reset_attr();
+	errno = 0;
 	rv = system(cp);
 	printf("\n");
 	if (rv < 0) {
@@ -408,7 +462,7 @@ char	*cp;	/* system command string */
  *
  * Return new position in string or NULL if error.
  */
-char	*
+char *
 parse_var2(vp, cp)
 long	*vp;	/* pointer to returned variable in Mathomatic internal format */
 char	*cp;	/* pointer to variable name string */
@@ -448,6 +502,24 @@ int	i;	/* command table index of command */
 	return rows;
 }
 
+#if	!LIBRARY
+int
+read_examples(cpp)
+char	**cpp;
+{
+	int	i;
+	char	*cp;
+
+	for (i = 0; cpp[i]; i++) {
+		cp = strdup(cpp[i]);
+		if (!display_process(cp))
+			return false;
+		free(cp);
+	}
+	return true;
+}
+#endif
+
 /*
  * The help command.
  */
@@ -483,14 +555,28 @@ next_argument:
 			}
 			return true;
 		}
+		if (strncasecmp(cp, "examples", cp1 - cp) == 0) {
+#if	LIBRARY
+			error(_("Examples not available."));
+			return false;
+#else
+			return read_examples(example_strings);
+#endif
+		}
+		if (strncasecmp(cp, "copyright", cp1 - cp) == 0 || strncasecmp(cp, "license", cp1 - cp) == 0) {
+			P("Copyright and License for Mathomatic");
+			P("------------------------------------\n");
+
+			fprintf(gfp, "%s", license_string);
+			return true;
+		}
 		if (strncasecmp(cp, "bugs", cp1 - cp) == 0) {
-			P("Report bugs on the Ubuntu Launchpad website:");
+			P("Report bugs on the Launchpad website:");
 			P("http://launchpad.net/mathomatic\n");
 
-			P("Please include version information from the version command:\n");
+			P("Please include the following information (from the version command):\n");
 
-			version_report();
-			return true;
+			return version_report();
 		}
 		if (strncasecmp(cp, "usage", cp1 - cp) == 0) {
 			P("Mathomatic Command Usage Syntax");
@@ -506,6 +592,32 @@ next_argument:
 			}
 			return true;
 		}
+		if (strncasecmp(cp, "geometry", cp1 - cp) == 0) {
+			P("Commonly used standard (Euclidean) geometric formulas");
+			P("-----------------------------------------------------");
+			P("Triangle of base \"b\" and height \"h\":");
+			P("    area = b*h/2\n");
+
+			P("Rectangle of length \"l\" and width \"w\":");
+			P("    area = l*w                    perimeter = 2*l + 2*w\n");
+
+			P("Trapezoid of parallel sides \"a\" and \"b\", and \"d\" distance between them:");
+			P("    area = d*(a + b)/2\n");
+
+			P("Circle of radius \"r\":");
+			P("    area = pi*r^2                 perimeter = 2*pi*r");
+			P("    area = perimeter*r/2\n");
+
+			P("Rectangular solid of length \"l\", width \"w\", and height \"h\":");
+			P("    volume = l*w*h                surface_area = 2*l*w + 2*l*h + 2*w*h\n");
+
+			P("Sphere of radius \"r\":");
+			P("    volume = 4/3*pi*r^3           surface_area = 4*pi*r^2\n");
+
+			P("Convex polygon of \"n\" sides, sum of all interior angles formula:");
+			P("    sum = (n - 2)*180 degrees     sum = (n - 2)*pi radians");
+			return true;
+		}
 		if (strncasecmp(cp, "expressions", cp1 - cp) == 0 || strncasecmp(cp, "equations", cp1 - cp) == 0) {
 			P("To enter an expression or equation, simply type it in at the prompt.");
 			P("Operators have precedence decreasing as indicated:\n");
@@ -513,8 +625,8 @@ next_argument:
 			P("    - negate (highest precedence)");
 			P("    ! factorial (gamma function)");
 			P("    ** or ^ power (exponentiation)");
-			P("    * multiply        / divide          % modulus         // integral divide");
-			P("    + add             - subtract");
+			P("    * multiply      / divide        % modulus       // integral divide");
+			P("    + add           - subtract");
 			P("    = equate (lowest precedence)\n");
 
 			P("Variables consist of any combination of letters, digits, and underscores (_).");
@@ -522,9 +634,9 @@ next_argument:
 
 			P("    e or e# - the universal constant e (2.7182818284...)");
 			P("    pi or pi# - the universal constant pi (3.1415926535...)");
-			P("    i or i# - the imaginary number (square root of -1)");
-			P("The above constants may also be used most anywhere variables are required.");
-			P("    sign, sign1, sign2, sign3, ... - variables that can only be +1 or -1");
+			P("    i or i# - the imaginary unit (square root of -1)");
+			P("The above constants may also be used anywhere variables are required.");
+			P("    sign, sign1, sign2, ... - variables that can only be +1 or -1");
 			P("    integer - variable that can only be an integer value");
 			P("    inf - floating point infinity constant (not a variable)\n");
 
@@ -550,11 +662,12 @@ next_argument:
 	}
 	/* default help text: */
 	P("Mathomatic is a Computer Algebra System (CAS) and calculator program.");
-	P("Type \"help bugs\" for info on how to report bugs found in this program.");
+	P("To see example expressions and commands, type \"help examples\".");
 	P("Type \"help equations\" for help with entering expressions and equations.");
 	P("Type \"help all\" for a summary and syntax of all commands.");
 	P("Type \"help usage\" to display the syntax of all commands.");
-	P("\"help\" or \"?\" followed by command names will give info on those commands.\n");
+	P("Type \"help geometry\" for some commonly used geometric formulas.");
+	P("\"help\" or \"?\" followed by a command name will give info on that command.\n");
 
 	fprintf(gfp, "These are the %d commands:\n", ARR_CNT(com_list));
 	for (i = 0; i < ARR_CNT(com_list); i++) {
