@@ -1,6 +1,6 @@
 /*
- * Standard include files for Mathomatic.
- * Automatically includes all necessary include files for
+ * Standard C include files for Mathomatic.
+ * Automatically includes all necessary C include files for
  * any Mathomatic C source code.
  *
  * Copyright (C) 1987-2009 George Gesslein II.
@@ -9,25 +9,31 @@
 #define	true	1
 #define	false	0
 
-#if	LIBRARY
-#define	SILENT	true
-#undef	READLINE
+#if	HANDHELD
+#undef	UNIX		/* Don't include the extra Unix desktop features. */
 #endif
 
-#if	__CYGWIN__ && !CYGWIN
-#warning "Compiling under Cygwin without proper defines.  Please define CYGWIN"
+#if	LIBRARY
+#define	SILENT	true	/* Stop messages going to stdout. */
+#undef	READLINE	/* Readline shouldn't be included in the library code. */
+#endif
+
+#if 	CYGWIN
+#undef	UNIX
+#elif	__CYGWIN__
+#warning Compiling under Cygwin without proper defines.
+#warning Please define CYGWIN on the compiler command line with -DCYGWIN
 #endif
 
 /* Include files from /usr/include: */
 #include <stdio.h>
 #include <stdlib.h>
-#if	UNIX || CYGWIN
 #include <unistd.h>
-#endif
 #if	UNIX
 #include <libgen.h>
 #endif
 #include <limits.h>
+#include <float.h>
 #include <math.h>
 #include <setjmp.h>
 #include <ctype.h>
@@ -35,16 +41,18 @@
 #include <errno.h>
 #include <signal.h>
 #if	I18N
-#include <libintl.h>
+#include <libintl.h>	/* Mac OS X doesn't have libintl.h, so define "char *gettext();" then. */
+#include <locale.h>
 #endif
 #if	READLINE
+#include <curses.h>
 #include <readline/readline.h>
 #include <readline/history.h>
 #endif
 
 /* Include files from the current directory: */
-#include "license.h"	/* the current license for Mathomatic */
 #include "am.h"		/* the main include file for Mathomatic, contains tunable parameters */
 #include "complex.h"	/* floating point complex number arithmetic function prototypes */
-#include "proto.h"	/* global function prototypes */
+#include "proto.h"	/* global function prototypes made with cproto utility */
+#include "altproto.h"	/* backup global function prototypes */
 #include "externs.h"	/* global variable extern definitions */
