@@ -296,7 +296,7 @@ char	*cp;
 		} else {
 			printf(_("Integrating with respect to (%s)"), var_str);
 		}
-		printf(" and simplifying...\n");
+		printf(_(" and simplifying...\n"));
 	}
 #endif
 	partial_flag = false;
@@ -662,7 +662,7 @@ char	*cp;
 		}
 	}
 	if (singularity) {
-		warning(_("Singularity detected, result of numerical integration may be wrong."));
+		warning(_("Singularity detected, result of numerical integration might be wrong."));
 	}
 	my_strlcpy(prompt_str, _("Enter lower bound: "), sizeof(prompt_str));
 	if (!get_expr(tlhs, &n_tlhs)) {
