@@ -1,11 +1,7 @@
 /*
- * This file contains a test for the Mathomatic symbolic math library and API.
- * Refer to this, if you are going to use the Mathomatic code in other projects.
- *
- * To run this, compile and link with the Mathomatic symbolic math library
- * and create an executable.
- *
- * Written by George Gesslein II.
+ * This file contains the test/example program
+ * for the Mathomatic symbolic math library and API.
+ * Copy or refer to this, if you are going to use the Mathomatic code in other projects.
  */
 
 #include <stdio.h>
@@ -21,7 +17,8 @@ main(int argc, char **argv)
 	char		buf[1000];	/* buffer */
 
 	printf("Mathomatic library test program.\n");
-	if (!matho_init()) {
+	/* Initialize all global variables and arrays so that Mathomatic will work properly. */
+	if (!matho_init()) {	/* call this function exactly once in your program */
 		fprintf(stderr, "Not enough memory.\n");
 		exit(1);
 	}
@@ -36,9 +33,11 @@ main(int argc, char **argv)
 			printf("%s\n", ocp);
 			if (rv) {
 				free((void *) ocp);
+			} else {
+				printf("Error return.\n");
 			}
 		}
 	}
-	printf("\nEOF\n");
+	printf("\n");
 	exit(0);
 }
