@@ -10,8 +10,6 @@
 #define	EQUATE_STRING	" = "	/* string displayed between the LHS and RHS of equations */
 #define MODULUS_STRING	" % "	/* string displayed for the modulus operator */
 
-#define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } len += strlen(str); }
-
 static int	flist();
 static int	flist_recurse();
 
@@ -58,20 +56,22 @@ static int	cur_pos;	/* current position in the current line on the screen */
 /*
  * Reset terminal attributes function.
  * Turn color off if color mode is on.
- * Subsequent output will have no color.
+ * Subsequent output will have no color until the next call to set_color().
  */
 void
 reset_attr()
 {
-	cur_color = -1;
-	if (color_flag) {
+	if (color_flag && cur_color >= 0) {
 		if (html_flag) {
 			printf("</font>");
 		} else {
 			printf("\033[0m");
 		}
 	}
-	fflush(NULL);	/* flush all output */
+	cur_color = -1;
+	if (fflush(NULL)) {	/* flush all output */
+		perror("fflush(3) call failed");
+	}
 }
 
 /*
@@ -114,6 +114,27 @@ default_color()
 	set_color(0);
 }
 
+/*
+ * Trim the trailing zeros from a string, after the decimal point.
+ */
+static void
+trim_zeros(buf)
+char	*buf;
+{
+	int	j;
+
+	j = strlen(buf) - 1;
+	for (; j >= 0; j--) {
+		if (buf[j] == '0')
+			continue;
+		if (buf[j] == '.') {
+			buf[j+2] = '\0';
+		} else {
+			break;
+		}
+	}
+}
+
 /*
  * Display the expression or equation stored in equation space "n" in single-line format.
  * The equation space is not modified.
@@ -124,7 +145,7 @@ int
 list1_sub(n, export_flag)
 int	n;		/* equation space number */
 int	export_flag;	/* non-zero for exportable format (readable by other math programs) */
-			/* 1 for Maxima, 2 for other */
+			/* 1 for Maxima, 2 for other, 3 for Gnuplot */
 {
 	int	len = 0;
 
@@ -213,17 +234,20 @@ long	v;		/* variable to convert */
 int	lang_code;	/* language code */
 {
 	int		j;
-	char		*cp = NULL, buf[100];
+	char		*cp = NULL;
 
 	var_str[0] = '\0';
 	switch (v & VAR_MASK) {
 	case V_NULL:
-		return(0);
+		return(strlen(var_str));
 	case SIGN:
 		cp = "sign";
 		break;
 	case IMAGINARY:
 		switch (lang_code) {
+		case -3:
+			cp = "{0,1}";
+			break;
 		case -2:
 			cp = "i";
 			break;
@@ -237,12 +261,15 @@ int	lang_code;	/* language code */
 			cp = "1j";
 			break;
 		default:
-			cp = "I";
+			cp = "1.0i";
 			break;
 		}
 		break;
 	case V_E:
 		switch (lang_code) {
+		case -3:
+			cp = "exp(1.0)";
+			break;
 		case -1:
 			cp = "%e";
 			break;
@@ -296,11 +323,11 @@ int	lang_code;	/* language code */
 		break;
 	}
 	if (cp) {
-		my_strlcpy(var_str, cp, sizeof(var_str));
 		j = (v >> VAR_SHIFT) & SUBSCRIPT_MASK;
 		if (j) {
-			snprintf(buf, sizeof(buf), "%d", j - 1);
-			strcat(var_str, buf);
+			snprintf(var_str, sizeof(var_str), "%s%d", cp, j - 1);
+		} else {
+			my_strlcpy(var_str, cp, sizeof(var_str));
 		}
 	} else {
 		my_strlcpy(var_str, "bad_variable", sizeof(var_str));
@@ -320,97 +347,9 @@ list_proc(p1, n, export_flag)
 token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 int		export_flag;	/* flag for exportable format (usually false) */
-				/* 1 for Maxima, 2 for other */
+				/* 1 for Maxima, 2 for other, 3 for Gnuplot */
 {
-	int	i, j, k, i1;
-	int	min1;
-	int	cur_level;
-	char	*cp;
-	int	len = 0;
-
-	cur_level = min1 = min_level(p1, n);
-	for (i = 0; i < n; i++) {
-		j = cur_level - p1[i].level;
-		k = abs(j);
-		for (i1 = 1; i1 <= k; i1++) {
-			if (j > 0) {
-				cur_level--;
-				len += fprintf(gfp, ")");
-				set_color(cur_level-min1);
-			} else {
-				cur_level++;
-				set_color(cur_level-min1);
-				len += fprintf(gfp, "(");
-			}
-		}
-		switch (p1[i].kind) {
-		case CONSTANT:
-			if (p1[i].token.constant == 0.0) {
-				p1[i].token.constant = 0.0; /* fix -0 */
-			}
-			if (high_prec || export_flag) {
-				if (p1[i].token.constant < 0.0) {
-					len += fprintf(gfp, "(%.20g)", p1[i].token.constant);
-				} else {
-					len += fprintf(gfp, "%.20g", p1[i].token.constant);
-				}
-			} else if (finance_option) {
-				if (p1[i].token.constant < 0.0) {
-					len += fprintf(gfp, "(%.*f)", finance_option, p1[i].token.constant);
-				} else {
-					len += fprintf(gfp, "%.*f", finance_option, p1[i].token.constant);
-				}
-			} else {
-				len += fprintf(gfp, "%.*g", precision, p1[i].token.constant);
-			}
-			break;
-		case VARIABLE:
-			list_var(p1[i].token.variable, 0 - export_flag);
-			len += fprintf(gfp, "%s", var_str);
-			break;
-		case OPERATOR:
-			cp = _("(unknown operator)");
-			switch (p1[i].token.operatr) {
-			case PLUS:
-				cp = " + ";
-				break;
-			case MINUS:
-				cp = " - ";
-				break;
-			case TIMES:
-				cp = "*";
-				break;
-			case DIVIDE:
-				cp = "/";
-				break;
-			case IDIVIDE:
-				cp = "//";
-				break;
-			case MODULUS:
-				cp = MODULUS_STRING;
-				break;
-			case POWER:
-				if (power_starstar) {
-					cp = "**";
-				} else {
-					cp = "^";
-				}
-				break;
-			case FACTORIAL:
-				cp = "!";
-				i++;
-				break;
-			}
-			len += fprintf(gfp, "%s", cp);
-			break;
-		}
-	}
-	for (j = cur_level - min1; j > 0;) {
-		len += fprintf(gfp, ")");
-		j--;
-		set_color(j);
-	}
-	return len;
+	return list_string_sub(p1, n, true, NULL, export_flag);
 }
 
 /*
@@ -430,10 +369,10 @@ int	export_flag;	/* flag for exportable format (usually false) */
 	if (n_lhs[n] <= 0) {
 		return NULL;
 	}
-	len = list_string_sub(lhs[n], n_lhs[n], NULL, export_flag);
+	len = list_string(lhs[n], n_lhs[n], NULL, export_flag);
 	if (n_rhs[n]) {
 		len += strlen(EQUATE_STRING);
-		len += list_string_sub(rhs[n], n_rhs[n], NULL, export_flag);
+		len += list_string(rhs[n], n_rhs[n], NULL, export_flag);
 	}
 	len += 2;
 	cp = (char *) malloc(len);
@@ -441,10 +380,10 @@ int	export_flag;	/* flag for exportable format (usually false) */
 		error(_("Out of memory (can't malloc(3))."));
 		return NULL;
 	}
-	list_string_sub(lhs[n], n_lhs[n], cp, export_flag);
+	list_string(lhs[n], n_lhs[n], cp, export_flag);
 	if (n_rhs[n]) {
 		strcat(cp, EQUATE_STRING);
-		list_string_sub(rhs[n], n_rhs[n], &cp[strlen(cp)], export_flag);
+		list_string(rhs[n], n_rhs[n], &cp[strlen(cp)], export_flag);
 	}
 	if (export_flag == 1) {
 		strcat(cp, ";");
@@ -470,14 +409,14 @@ int		export_flag;
 	if (n <= 0) {
 		return NULL;
 	}
-	len = list_string_sub(p1, n, NULL, export_flag);
+	len = list_string(p1, n, NULL, export_flag);
 	len++;
 	cp = (char *) malloc(len);
 	if (cp == NULL) {
 		error(_("Out of memory (can't malloc(3))."));
 		return NULL;
 	}
-	list_string_sub(p1, n, cp, export_flag);
+	list_string(p1, n, cp, export_flag);
 	return cp;
 }
 
@@ -485,25 +424,44 @@ int		export_flag;
  * Convert an expression to a text string and store in "string" if
  * "string" is not NULL.  "string" need not be initialized,
  * but must be long enough to contain the expression.
+ * To find the storage size needed, call with "string" set to NULL first.
  *
  * Return length (number of characters).
  */
 int
-list_string_sub(p1, n, string, export_flag)
+list_string(p1, n, string, export_flag)
 token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
-char		*string;	/* buffer or NULL pointer */
+char		*string;	/* buffer to save output to or NULL pointer */
 int		export_flag;
 {
+	return list_string_sub(p1, n, false, string, export_flag);
+}
+
+int
+list_string_sub(p1, n, outflag, string, export_flag)
+token_type	*p1;		/* expression pointer */
+int		n;		/* length of expression */
+int		outflag;	/* if true, output to gfp */
+char		*string;	/* buffer to save output to or NULL pointer */
+int		export_flag;
+{
+#define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } if (outflag) { fprintf(gfp, "%s", str); } len += strlen(str); }
+
 	int	i, j, k, i1;
 	int	min1;
 	int	cur_level;
 	char	*cp;
 	int	len = 0;
-	char	buf[500];
+	char	buf[500], buf2[500];
+	int	export_precision;
 
 	if (string)
 		string[0] = '\0';
+	if (high_prec)
+		export_precision = 20;
+	else
+		export_precision = DBL_DIG;
 	cur_level = min1 = min_level(p1, n);
 	for (i = 0; i < n; i++) {
 		j = cur_level - p1[i].level;
@@ -512,8 +470,12 @@ int		export_flag;
 			if (j > 0) {
 				cur_level--;
 				APPEND(")");
+				if (outflag && !export_flag)
+					set_color(cur_level-min1);
 			} else {
 				cur_level++;
+				if (outflag && !export_flag)
+					set_color(cur_level-min1);
 				APPEND("(");
 			}
 		}
@@ -522,22 +484,28 @@ int		export_flag;
 			if (p1[i].token.constant == 0.0) {
 				p1[i].token.constant = 0.0; /* fix -0 */
 			}
-			if (high_prec || export_flag) {
-				if (p1[i].token.constant < 0.0) {
-					snprintf(buf, sizeof(buf), "(%.20g)", p1[i].token.constant);
-				} else {
-					snprintf(buf, sizeof(buf), "%.20g", p1[i].token.constant);
-				}
+			if (export_flag == 3) {
+				snprintf(buf, sizeof(buf), "%#.*g", DBL_DIG, p1[i].token.constant);
+				trim_zeros(buf);
+			} else if (export_flag || high_prec) {
+				snprintf(buf, sizeof(buf), "%.*g", export_precision, p1[i].token.constant);
 			} else if (finance_option) {
-				if (p1[i].token.constant < 0.0) {
-					snprintf(buf, sizeof(buf), "(%.*f)", finance_option, p1[i].token.constant);
-				} else {
-					snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
-				}
+				snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
 			} else {
-				snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
+				if (p1[i].token.constant < 0.0 && !negate_highest_precedence) {
+					snprintf(buf, sizeof(buf), "(%.*g)", precision, p1[i].token.constant);
+				} else {
+					snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
+				}
+				APPEND(buf);
+				break;
+			}
+			if (p1[i].token.constant < 0.0) {
+				snprintf(buf2, sizeof(buf2), "(%s)", buf);
+				APPEND(buf2);
+			} else {
+				APPEND(buf);
 			}
-			APPEND(buf);
 			break;
 		case VARIABLE:
 			list_var(p1[i].token.variable, 0 - export_flag);
@@ -565,7 +533,7 @@ int		export_flag;
 				cp = MODULUS_STRING;
 				break;
 			case POWER:
-				if (power_starstar) {
+				if (power_starstar || export_flag == 3) {
 					cp = "**";
 				} else {
 					cp = "^";
@@ -580,8 +548,11 @@ int		export_flag;
 			break;
 		}
 	}
-	for (j = cur_level - min1; j > 0; j--) {
+	for (j = cur_level - min1; j > 0;) {
 		APPEND(")");
+		j--;
+		if (outflag && !export_flag)
+			set_color(j);
 	}
 	return len;
 }
@@ -598,9 +569,19 @@ int		n;		/* length of expression */
 	int	i;
 
 	for (i = 0; i < n; i++) {
-		if ((p1[i].kind == CONSTANT && fmod(p1[i].token.constant, 1.0))
-		    || (p1[i].kind == VARIABLE && p1[i].token.variable <= IMAGINARY)) {
-			return false;
+		switch (p1[i].kind) {
+		case CONSTANT:
+			if (fmod(p1[i].token.constant, 1.0)) {
+				return false;
+			}
+			break;
+		case VARIABLE:
+			if (p1[i].token.variable < IMAGINARY) {
+				return false;
+			}
+			break;
+		case OPERATOR:
+			break;
 		}
 	}
 	return true;
@@ -705,17 +686,8 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 			if (int_flag) {
 				snprintf(buf, sizeof(buf), "%.0f", equation[i].token.constant);
 			} else {
-				snprintf(buf, sizeof(buf), "%#.*g", MAX_PRECISION, equation[i].token.constant);
-				j = strlen(buf) - 1;
-				for (; j >= 0; j--) {
-					if (buf[j] == '0')
-						continue;
-					if (buf[j] == '.') {
-						buf[j+2] = '\0';
-					} else {
-						break;
-					}
-				}
+				snprintf(buf, sizeof(buf), "%#.*g", DBL_DIG, equation[i].token.constant);
+				trim_zeros(buf);
 			}
 			if (equation[i].token.constant < 0) {
 				len += fprintf(gfp, "(%s)", buf);
@@ -724,8 +696,12 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 			}
 			break;
 		case VARIABLE:
-			len += list_var(equation[i].token.variable, language + 1);
-			fprintf(gfp, "%s", var_str);
+			if (int_flag && language <= 1 && equation[i].token.variable == IMAGINARY) {
+				len += fprintf(gfp, "1i");
+			} else {
+				list_var(equation[i].token.variable, language + 1);
+				len += fprintf(gfp, "%s", var_str);
+			}
 			break;
 		case OPERATOR:
 			cp = _("(unknown operator)");
@@ -823,7 +799,13 @@ make_smaller:
 	width = max(len + len2, len3);
 	if (screen_columns && gfp == stdout && width >= screen_columns) {
 		/* output too wide to fit screen, output in single-line format */
-		return list1_sub(n, false);
+		width = list1_sub(n, false);
+#if	CYGWIN
+		fprintf(gfp, "\r\n");	/* Be consistent with list1_sub() output. */
+#else
+		fprintf(gfp, "\n");
+#endif
+		return width;
 	}
 	fprintf(gfp, "\n");
 	for (cur_line = max_line; cur_line >= min_line; cur_line--) {
@@ -898,7 +880,7 @@ int		*highp, *lowp;
 	int	len_div;
 	int	level;
 	int	start_level;
-	int	oflag;
+	int	oflag, html_out;
 	int	len = 0, len1, len2;
 	int	high, low;
 	char	buf[500];
@@ -911,6 +893,7 @@ int		*highp, *lowp;
 		return 0;
 	}
 	oflag = (out_flag && line == cur_line);
+	html_out = (html_flag && gfp == stdout);
 	if (oflag) {
 		set_color(cur_level-1);
 		for (; cur_pos < pos; cur_pos++) {
@@ -992,8 +975,8 @@ check_again:
 			if (oflag) {
 				set_color(level-1);
 				for (j = 0; j < len_div; j++) {
-					if (html_flag && gfp == stdout) {
-						fprintf(gfp, "&mdash;");
+					if (html_out) {
+						fprintf(gfp, "&minus;");
 					} else {
 						fprintf(gfp, "-");
 					}
@@ -1024,7 +1007,11 @@ check_again:
 					len += snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
 				}
 			} else {
-				len += snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
+				if (p1[i].token.constant < 0.0 && !negate_highest_precedence) {
+					len += snprintf(buf, sizeof(buf), "(%.*g)", precision, p1[i].token.constant);
+				} else {
+					len += snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
+				}
 			}
 			if (oflag)
 				fprintf(gfp, "%s", buf);
@@ -1035,14 +1022,14 @@ check_again:
 				fprintf(gfp, "%s", var_str);
 			break;
 		case OPERATOR:
-			/* len must be incremented here by the number of columns the display of the operator takes */
+/* "len" must be incremented here by the number of columns the display of the operator takes */
 			switch (p1[i].token.operatr) {
 			case PLUS:
 				cp = " + ";
 				len += 3;
 				break;
 			case MINUS:
-				if (html_flag && gfp == stdout) {
+				if (html_out) {
 					cp = " &minus; ";
 				} else {
 					cp = " - ";
@@ -1050,7 +1037,7 @@ check_again:
 				len += 3;
 				break;
 			case TIMES:
-				if (html_flag && gfp == stdout) {
+				if (html_out) {
 					cp = "&middot;";
 				} else {
 					cp = "*";
@@ -1070,6 +1057,25 @@ check_again:
 				len += strlen(cp);
 				break;
 			case POWER:
+#if	false
+				if (html_out
+				    && p1[i-1].level == p1[i].level
+				    && p1[i+1].level == p1[i].level
+				    && p1[i+1].kind == CONSTANT) {
+					if (p1[i+1].token.constant == 2.0) {
+						cp = "&sup2;";
+						len++;
+						i++;
+						break;
+					}
+					if (p1[i+1].token.constant == 3.0) {
+						cp = "&sup3;";
+						len++;
+						i++;
+						break;
+					}
+				}
+#endif
 				if (power_starstar) {
 					cp = "**";
 					len += 2;
