@@ -39,7 +39,8 @@
  * to adjust memory usage.
  */
 
-#if	!LIBRARY
+#if	!LIBRARY	/* This comments out this whole file if using as symbolic math library. */
+
 #include "includes.h"
 #if	UNIX || CYGWIN
 #include <sys/ioctl.h>
@@ -49,8 +50,6 @@
 #include <fenv.h>
 #endif
 
-static char	rc_file[PATH_MAX];
-
 /*
  * Display invocation usage info.
  */
@@ -66,6 +65,7 @@ usage()
 	printf(_("  -m number     Specify a memory size multiplier.\n"));
 	printf(_("  -q            Set quiet mode (don't display prompts).\n"));
 	printf(_("  -r            Disable readline.\n"));
+	printf(_("  -s level      Set enforced security level for session.\n"));
 	printf(_("  -t            Set test mode.\n"));
 	printf(_("  -u            Set unbuffered output.\n"));
 	printf(_("  -v            Display version information, then exit.\n"));
@@ -88,25 +88,30 @@ char	**argv;
 	double		new_size;
 	int		coption = false, boption = false, wide_flag = false;
 
+#if	I18N
+	/* Initialize internationalization so that messages are in the right language. */
+	setlocale(LC_ALL, "");
+	textdomain(prog_name);
+	bindtextdomain(prog_name, LOCALEDIR);
+#endif
+
 #if	CYGWIN
 	dir_path = strdup(dirname_win(argv[0]));	/* set dir_path to this executable's directory */
 #endif
 	/* initialize the global variables */
 	init_gvars();
 	gfp = stdout;
-#if	READLINE
-	if ((cp = getenv("HOME")) && prog_name) {
-		snprintf(history_filename_storage, sizeof(history_filename_storage), "%s/.%s_history", cp, prog_name);
-		history_filename = history_filename_storage;
-	}
-	using_history();		/* initialize readline history */
-	stifle_history(500);		/* maximum of 500 entries */
-	rl_inhibit_completion = true;	/* turn off readline file name completion */
-#endif
 
 	/* process command line options */
-	while ((i = getopt(argc, argv, "bqrtchuvwxm:")) >= 0) {
+	while ((i = getopt(argc, argv, "s:bqrtchuvwxm:")) >= 0) {
 		switch (i) {
+		case 's':
+#if	SECURE
+			fprintf(stderr, _("%s: Compiled for maximum security, therefore no need for setting security level.\n"), prog_name);
+#else
+			security_level = atoi(optarg);
+#endif
+			break;
 		case 'w':
 			wide_flag = true;
 			break;
@@ -155,17 +160,35 @@ char	**argv;
 		}
 	}
 	if (n_tokens < 100) {
-		fprintf(stderr, _("%s: Improper equation space size.\n"), prog_name);
+		fprintf(stderr, _("%s: Expression array size too small.\n"), prog_name);
 		exit(2);
 	}
 	if (!init_mem()) {
 		fprintf(stderr, _("%s: Not enough memory.\n"), prog_name);
 		exit(2);
 	}
-#if	READLINE && !SECURE
-	if (readline_enabled) {
+#if	READLINE
+	if (readline_enabled) {	/* readline_enabled flag must not change after this */
+		if ((cp = getenv("HOME")) && prog_name) {
+			snprintf(history_filename_storage, sizeof(history_filename_storage), "%s/.%s_history", cp, prog_name);
+			history_filename = history_filename_storage;
+		}
+		rl_initialize();		/* initialize readline */
+		using_history();		/* initialize readline history */
+		stifle_history(500);		/* maximum of 500 entries */
+		rl_inhibit_completion = true;	/* turn off readline file name completion */
+#if	!NO_COLOR && !LIBRARY	/* not 100% tested and this might confuse the user with the -c toggle */
+		if (!html_flag) {		/* If doing ANSI color: */
+			color_flag = (tigetnum("colors") >= 8);	/* autoset color output mode. */
+		}
+#endif
+#if	!SECURE
 		read_history(history_filename);	/* restore readline history of previous session */
+#endif
 	}
+#endif
+#if	UNIX || CYGWIN
+	get_screen_size();
 #endif
 	if (html_flag) {
 		printf("<pre>\n");
@@ -173,10 +196,13 @@ char	**argv;
 	if (!test_mode && !quiet_mode) {
 #if	SECURE
 		printf(_("Secure "));
+#else
+		if (security_level >= 2)
+			printf(_("Secure "));
 #endif
-		printf(_("Mathomatic version %s (www.mathomatic.org)\n"), VERSION);
-		printf(_("Copyright (C) 1987-2009 George Gesslein II.\n"));
-		printf(_("%d equation spaces available, %ld kilobytes per equation space.\n"),
+		printf("Mathomatic version %s (www.mathomatic.org)\n", VERSION);
+		printf("Copyright (C) 1987-2009 George Gesslein II.\n");
+		printf(_("%d equation spaces available, %ld Kbytes per equation space.\n"),
 		    N_EQUATIONS, (long) n_tokens * sizeof(token_type) * 2L / 1000L);
 	}
 #if	!SECURE
@@ -188,24 +214,21 @@ char	**argv;
 	if (wide_flag) {
 		screen_columns = 0;
 		screen_rows = 0;
-#if	UNIX || CYGWIN
-	} else {
-		get_screen_size();
-#endif
 	}
-	if (test_mode) {
-		color_flag = false;
-	} else if (coption & 1) {
+	if (coption & 1) {
 		color_flag = !color_flag;
 	}
 	if (boption) {
-		bold_colors = true;
 		color_flag = true;
+		bold_colors = true;
+	}
+	if (test_mode) {
+		color_flag = false;
 	}
 	if (!quiet_mode && color_flag) {
 		printf(_("%s%s color mode enabled"), html_flag ? "HTML" : "ANSI", bold_colors ? " bold" : "");
 		if (!boption) {
-			printf(_("; disable with the -c option.\n"));
+			printf(_("; disable with the -c option or \"set no color\".\n"));
 		} else {
 			printf(".\n");
 		}
@@ -233,23 +256,36 @@ char	**argv;
 		}
 #if	!SECURE
 		/* read in files specified on the command line, exit if error */
-		for (i = optind; i < argc; i++) {
-			if (!read_cmd(argv[i])) {
+		for (i = optind; i < argc && argv[i]; i++) {
+			if (strcmp(argv[i], "-") == 0) {
+				read_stdin();
+			} else if (!read_cmd(argv[i])) {
+				fprintf(stderr, _("Read of file \"%s\" failed.\n"), argv[i]);
 				exit_program(1);
 			}
 		}
 #endif
 	}
-	/* main input/output loop */
+	read_stdin();		/* main input/output loop */
+	exit_program(0);	/* exit Mathomatic, doesn't return */
+	return(0);		/* so the compiler doesn't complain */
+}
+
+/*
+ * Repeatedly read a line of text from standard input and process the expression or command.
+ */
+void
+read_stdin()
+{
+	char	*cp = NULL;
+
 	for (;;) {
 		default_color();
-		snprintf(prompt_str, sizeof(prompt_str), "%d%s", cur_equation + 1, html_flag ? HTML_PROMPT : PROMPT);
+		snprintf(prompt_str, sizeof(prompt_str), "%d%s", cur_equation + 1, html_flag ? HTML_PROMPT_STR : PROMPT_STR);
 		if ((cp = get_string((char *) tlhs, n_tokens * sizeof(token_type))) == NULL)
 			break;
 		process(cp);
 	}
-	exit_program(0);
-	return(0);	/* so the compiler doesn't complain */
 }
 
 /*
@@ -286,14 +322,17 @@ set_signals()
 
 #if	CYGWIN
 /*
- * dirname(3) function for Windows.
+ * dirname(3) function for Microsoft Windows.
+ * dirname(3) strips the non-directory suffix from a filename.
  */
-char	*
+char *
 dirname_win(cp)
-char	*cp;
+char	*cp;	/* string containing filename to modify */
 {
 	int	i;
 
+	if (cp == NULL)
+		return(".");
 	i = strlen(cp);
 	while (i >= 0 && cp[i] != '\\' && cp[i] != '/')
 		i--;
@@ -306,9 +345,10 @@ char	*cp;
 
 #if	!SECURE
 /*
- * Load set options from "~/.mathomaticrc".
+ * Load set options from startup file "~/.mathomaticrc".
  *
- * Return false if error encountered.
+ * Return false if there was an error reading the startup file,
+ * otherwise return true.
  */
 int
 load_rc()
@@ -330,7 +370,6 @@ load_rc()
 	}
 #endif
 	if (fp == NULL) {
-		rc_file[0] = '\0';
 		return true;
 	}
 	while ((cp = fgets(buf, sizeof(buf), fp)) != NULL) {
@@ -345,6 +384,8 @@ load_rc()
 
 /*
  * Floating point exception handler.
+ * Floating point exceptions are currently ignored.
+ *
  * I do not know the proper function calls that should be here to make the TRAP_FPERRORS define work.
  * Currently it goes in an endless loop when an FP exception occurs and TRAP_FPERRORS is true,
  * because when this function returns, the exception immediately happens again.
@@ -373,7 +414,7 @@ int	sig;
 	case 0:
 	case 1:
 		/* wait for graceful abort */
-		printf(_("\nUser interrupt signal received.\n"));
+		printf(_("\nUser interrupt signal received; Three times quits.\n"));
 		return;
 	case 2:
 		printf(_("\nPress Control-C once more to quit program.\n"));
@@ -399,6 +440,8 @@ int	sig;
 #if	UNIX || CYGWIN
 /*
  * Get the screen (window) width and height from the operating system.
+ *
+ * Sets the global integers screen_columns and screen_rows.
  */
 void
 get_screen_size()
@@ -408,10 +451,10 @@ get_screen_size()
 	ws.ws_col = 0;
 	ws.ws_row = 0;
 	if (ioctl(1, TIOCGWINSZ, &ws) >= 0) {
-		if (ws.ws_col > 0 && screen_columns) {
+		if (ws.ws_col > 0) {
 			screen_columns = ws.ws_col;
 		}
-		if (ws.ws_row > 0 && screen_rows) {
+		if (ws.ws_row > 0) {
 			screen_rows = ws.ws_row;
 		}
 	}
@@ -424,7 +467,8 @@ void
 resizehandler(sig)
 int	sig;
 {
-	get_screen_size();
+	if (screen_columns)
+		get_screen_size();
 }
 #endif
 
@@ -441,10 +485,12 @@ int	exit_value;	/* zero if OK, non-zero indicates error return */
 	}
 #if	READLINE && !SECURE
 	if (readline_enabled) {
-		write_history(history_filename);	/* save readline history */
+		if (write_history(history_filename)) {	/* save readline history */
+			fprintf(stderr, _("%s: Unable to save readline history into file \"%s\".\n"), prog_name, history_filename);
+		}
 	}
 #endif
-	if (!quiet_mode)
+	if (exit_value == 0 && !quiet_mode && !html_flag)
 		printf(_("Thank you for using Mathomatic!\n"));
 	exit(exit_value);
 }
