@@ -165,16 +165,17 @@ int		*np;		/* pointer to expression length */
  * This is a simple, non-recursive mathematical expression parser.
  * To parse, the character string is sequentially read and stored
  * in the internal storage format.
- * The maximum length that can be parsed is the size of an equation side.
+ * The maximum length that can be parsed is the size of an equation side (n_tokens).
  * Any syntax and other errors are carefully reported and give a NULL return.
  *
  * Returns the new string position, or NULL if error.
  */
-char	*
-parse_section(equation, np, cp)
+char *
+parse_section(equation, np, cp, allow_space)
 token_type	*equation;	/* where the parsed expression is stored (equation side) */
 int		*np;		/* pointer to the returned parsed expression length */
 char		*cp;		/* string to parse */
+int		allow_space;	/* if false, any space characters terminate parsing */
 {
 	int		n = 0;			/* position in equation[] */
 	int		cur_level = 1;		/* current level of parentheses */
@@ -192,9 +193,6 @@ char		*cp;		/* string to parse */
 			error_huge();
 		}
 		switch (*cp) {
-		case ' ':
-		case '\t':
-			continue;
 		case '(':
 		case '{':
 			if (operand) {
@@ -221,6 +219,10 @@ char		*cp;		/* string to parse */
 				goto syntax_error;
 			}
 			continue;
+		case ' ':
+		case '\t':
+			if (allow_space)
+				continue;
 		case '=':
 		case ';':
 		case '\0':
@@ -463,16 +465,18 @@ syntax_error:
  * Returns the new string position, or NULL if error.
  * Currently, there can be no more to parse in the string when this returns.
  */
-char	*
+char *
 parse_equation(n, cp)
 int	n;	/* equation space number */
 char	*cp;	/* pointer to the beginning of the equation character string */
 {
+	if (cp == NULL)
+		return NULL;
 	if (!case_sensitive_flag) {
 		str_tolower(cp);
 	}
-	if ((cp = parse_section(lhs[n], &n_lhs[n], cp)) != NULL) {
-		if ((cp = parse_section(rhs[n], &n_rhs[n], cp)) != NULL) {
+	if ((cp = parse_section(lhs[n], &n_lhs[n], cp, true)) != NULL) {
+		if ((cp = parse_section(rhs[n], &n_rhs[n], cp, true)) != NULL) {
 			if (extra_characters(cp))
 				return NULL;
 			return cp;
@@ -487,7 +491,7 @@ char	*cp;	/* pointer to the beginning of the equation character string */
  * Returns the new string position, or NULL if error.
  * Currently, there can be no more to parse in the string when this returns.
  */
-char	*
+char *
 parse_expr(equation, np, cp)
 token_type	*equation;	/* where the parsed expression is stored (equation side) */
 int		*np;		/* pointer to the returned parsed expression length */
@@ -496,6 +500,8 @@ char		*cp;		/* string to parse */
 	int	i;
 	char	*cp1, *cp2;
 
+	if (cp == NULL)
+		return NULL;
 	if (!case_sensitive_flag) {
 		str_tolower(cp);
 	}
@@ -517,7 +523,7 @@ char		*cp;		/* string to parse */
 			return NULL;
 		}
 		if (i < 0 || i >= n_equations || n_lhs[i] <= 0) {
-			error(_("Equation not defined."));
+			error(_("No expression available in specified equation space."));
 			return NULL;
 		}
 		if (n_rhs[i]) {
@@ -533,7 +539,7 @@ char		*cp;		/* string to parse */
 #endif
 		return cp2;
 	}
-	if ((cp = parse_section(equation, np, cp)) != NULL) {
+	if ((cp = parse_section(equation, np, cp, true)) != NULL) {
 		if (extra_characters(cp))
 			return NULL;
 	}
@@ -549,7 +555,7 @@ char		*cp;		/* string to parse */
  * Return new string position if successful.
  * Display error message and return NULL on failure.
  */
-char	*
+char *
 parse_var(vp, cp)
 long	*vp;
 char	*cp;
@@ -585,6 +591,9 @@ char	*cp;
 	if (strcasecmp(buf, INFINITY_NAME) == 0) {
 		error(_("Infinity cannot be used as a variable."));
 		return(NULL);
+	} else if (strcasecmp(buf, "nan") == 0) {
+		error(_("NaN (Not a Number) cannot be directly entered."));
+		return(NULL);
 	} else if ((*strcmpfunc)(buf, "sign") == 0) {
 		vtmp = SIGN;
 	} else {
@@ -699,11 +708,9 @@ char	*cp;	/* input string */
 
 	point_flag = true;
 /* handle comments, newlines, and the DOS EOF character (control-Z) by truncating the string where found */
-	for (cp1 = cp; *cp1; cp1++) {
-		if (*cp1 == ';' || *cp1 == '\n' || *cp1 == '\r' || *cp1 == '\032') {
-			*cp1 = '\0';
-			break;
-		}
+	cp1 = strpbrk(cp, ";\n\r\032");
+	if (cp1) {
+		*cp1 = '\0';
 	}
 	remove_trailing_spaces(cp);
 /* set point_flag to false if non-printable characters encountered */
