@@ -34,7 +34,6 @@ Options:
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <libgen.h>
 #include <unistd.h>
 #include <ctype.h>
 #include <string.h>
@@ -60,7 +59,7 @@ Options:
 /* Maximum memory usage in bytes; can be set to any size; the larger, the faster: */
 #define BUFFER_SIZE	2000000
 #if	BUFFER_SIZE >= (INT_MAX / 2) || BUFFER_SIZE <= 0
-#error	"BUFFER_SIZE too large."
+#error BUFFER_SIZE too large.
 #endif
 
 void		generate_primes(void);
@@ -103,8 +102,6 @@ main(int argc, char *argv[])
 	int		i;
 	char		buf[1000];
 
-/*	prog_name = strdup(basename(argv[0])); */
-
 	start_value = -1.0;
 /* set the highest number this program will work with: */
 	max_integer = powl(10.0L, (long double) (LDBL_DIG));
@@ -255,7 +252,8 @@ elim_factor(long double arg)
 }
 
 /*
- * Generate and display consecutive prime numbers.
+ * Generate and display "number" consecutive prime numbers,
+ * from "start_value" to "end_value".
  */
 void
 generate_primes(void)
@@ -310,8 +308,9 @@ generate_primes(void)
 }
 
 /*
- * Parse a space or null terminated ASCII number in the string pointed to by "cp" and
- * return true with a floating point long double value in "*dp" if a valid integer,
+ * Parse a space or null terminated ASCII number in the string pointed to by "cp".
+ *
+ * Return true with a floating point long double value in "*dp" if a valid integer,
  * otherwise display an error message and return false.
  */
 int
@@ -319,7 +318,12 @@ get_long_double_int(char *cp, long double *dp)
 {
 	char	*cp1;
 
+	errno = 0;
 	*dp = strtold(cp, &cp1);
+	if (errno) {
+		perror(NULL);
+		return false;
+	}
 	if (cp == cp1) {
 		fprintf(stderr, "Invalid number.\n");
 		return false;
@@ -337,11 +341,11 @@ get_long_double_int(char *cp, long double *dp)
 		return false;
 	}
 	if (*dp > max_integer) {
-		fprintf(stderr, "Number is too large.\n");
+		fprintf(stderr, "Number is too large, maximum is %Lg.\n", max_integer);
 		return false;
 	}
 	if (*dp < 0.0 || fmodl(*dp, 1.0L) != 0.0) {
-		fprintf(stderr, "Number must be a positive integer.\n");
+		fprintf(stderr, "Number must be a positive integer or zero.\n");
 		return false;
 	}
 	return true;
@@ -375,9 +379,9 @@ test_pal(long double d, long double base)
 void
 usage(void)
 {
-	printf("Prime number generator version 1.0\n");
+	printf("\nPrime number generator version 1.0\n");
 	printf("Usage: %s [start [stop]] [\"twin\"] [\"pal\" [base]]\n\n", prog_name);
-	printf("Generate batches of consecutive prime numbers up to %Lg.\n", max_integer);
+	printf("Generate consecutive prime numbers from start to stop up to %Lg.\n", max_integer);
 	printf("If \"twin\" is specified, output only twin primes.\n");
 	printf("If \"pal\" is specified, output only palindromic primes.\n");
 	printf("The palindromic base may be specified, the default is base 10.\n");
@@ -387,9 +391,9 @@ usage(void)
 void
 usage2(void)
 {
-	printf("Prime number generator version 1.0\n");
+	printf("\nPrime number generator version 1.0\n");
 	printf("Usage: %s [options] [start [stop]]\n\n", prog_name);
-	printf("Generate batches of consecutive prime numbers up to %Lg.\n", max_integer);
+	printf("Generate consecutive prime numbers from start to stop up to %Lg.\n", max_integer);
 	printf("Options:\n");
 	printf("  -t               Output only twin primes.\n");
 	printf("  -p base          Output only palindromic primes.\n");
