@@ -7,9 +7,11 @@
 #include "includes.h"
 
 /*
- * The following defines the maximum number of terms on the same level that can be compared.
- * Most of the stack usage is related this value,
- * this many pointers are pushed onto the stack with each level of parentheses while comparing two expressions.
+ * MAX_COMPARE_TERMS defines the maximum number of terms
+ * on the same level of parentheses that can be compared.
+ * Most of the stack usage is related this value.
+ * Space for this many pointers is reserved on the stack
+ * with each level of parentheses while comparing two expressions.
  */
 #define	MAX_COMPARE_TERMS	(DEFAULT_N_TOKENS / 6)
 
@@ -30,6 +32,7 @@ token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to length of equation side */
 {
 	if (*np <= 0 || (*np & 1) != 1) {
+		printf("Bad expression size = %d.\n", *np);
 		error_bug("Internal error: organize() called with bad expression size.");
 	}
 	if (*np > n_tokens) {
@@ -143,7 +146,7 @@ int		loc, level, *elocp;
 }
 
 /*
- * The most basic simplification loop.
+ * The quickest, most basic simplification loop.
  * Just does constant simplification.
  */
 void
@@ -152,7 +155,7 @@ token_type	*equation;	/* pointer to the beginning of equation side to simplify *
 int		*np;		/* pointer to length of equation side */
 {
 	if (abort_flag) {
-		/* Control-C pressed, return to main prompt and leave unsimplified */
+		/* Control-C pressed, gracefully return to main prompt and leave unsimplified */
 		abort_flag = false;
 		longjmp(jmp_save, 13);
 	}
@@ -228,7 +231,7 @@ simp_side(equation, np)
 token_type	*equation;	/* pointer to the beginning of equation side to simplify */
 int		*np;		/* pointer to length of equation side */
 {
-	simp_ssub(equation, np, 0L, /* 0.0 */ 1.0, true, true, 1);
+	simp_ssub(equation, np, 0L, 1.0, true, true, 2 /* was 1 */);
 }
 
 /*
@@ -243,7 +246,7 @@ int		zsolve;		/* true for solving for zero */
 	elim_loop(equation, np);
 	simp_constant_power(equation, np);
 	do {
-		simp_ssub(equation, np, 0L, 0.0, !zsolve, true, 2);
+		simp_ssub(equation, np, 0L, 0.0, !zsolve, true, 6 /* was 2 */);
 	} while (super_factor(equation, np, 0));
 }
 
@@ -272,9 +275,7 @@ int		*np;		/* pointer to equation side length */
 long		v;		/* variable to factor */
 {
 	if (v == IMAGINARY) {
-		do {
-			elim_loop(equation, np);
-		} while (complex_root_simp(equation, np));
+		approximate_complex_roots(equation, np);
 	}
 	do {
 		do {
@@ -302,7 +303,8 @@ int		*np;
 	uf_simp(equation, np);
 	factorv(equation, np, IMAGINARY);
 	simp_side(equation, np);
-	uf_simp(equation, np);
+	make_fractions(equation, np);
+	uf_tsimp(equation, np);
 	approximate_roots = false;
 }
 
@@ -423,9 +425,10 @@ sort_type	*p1, *p2;
  * Factors variables in order: "sign" variables first, then by frequency.
  */
 void
-simpb_side(equation, np, power_flag, fc_level)
+simpb_side(equation, np, uf_power_flag, power_flag, fc_level)
 token_type	*equation;	/* pointer to the beginning of equation side */
 int		*np;		/* pointer to length of equation side */
+int		uf_power_flag;	/* uf_allpower() flag */
 int		power_flag;	/* factor_power() flag */
 int		fc_level;	/* factor constants code, passed to factor_constants() */
 {
@@ -435,9 +438,7 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
 	sort_type	va[MAX_VARS];	/* array of all real variables found in equation side */
 
 	simp_loop(equation, np);
-	if (true) {
-		uf_pplus(equation, np);
-	} else {
+	if (uf_power_flag) {
 		uf_allpower(equation, np);
 	}
 	last_v = 0;
@@ -500,17 +501,19 @@ int		*np;		/* pointer to length of equation side */
 {
 	if (*np == 0)
 		return;
+
 	do {
 		do {
 			do {
-				simp_ssub(equation, np, 0L, 1.0, false, true, 1);
+				simp_ssub(equation, np, 0L, 1.0, false, true, 5);
 			} while (poly_gcd_simp(equation, np));
 		} while (uf_power(equation, np));
 	} while (super_factor(equation, np, 2));
-	display_fractions_and_group(equation, np);
+
+	make_fractions(equation, np);
 	uf_tsimp(equation, np);
-	poly_factor(equation, np);
-	simpb_side(equation, np, true, 2);
+	poly_factor(equation, np, true);
+	simpb_side(equation, np, true, false, 2);
 }
 
 /*
@@ -535,17 +538,21 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	if (*np <= 1)	/* no need to simplify a single constant or variable */
 		return;
 	side_debug(2, equation, *np);
-	simpb_side(equation, np, true, 1);
-	rationalize(equation, np);
+	simpb_side(equation, np, false, true, 1);
+	if (rationalize_denominators) {
+		rationalize(equation, np);
+	}
 	unsimp_power(equation, np);
 	uf_times(equation, np);
 	simp_loop(equation, np);
+
 /* Here is the only place in Mathomatic that we do modulus (%) simplification: */
 	uf_pplus(equation, np);
 	uf_repeat(equation, np);
 	do {
 		elim_loop(equation, np);
 	} while (mod_simp(equation, np));
+
 /* Here we try to simplify out unnecessary negative constants and imaginary numbers: */
 	simp_i(equation, np);
 	unsimp_power(equation, np);
@@ -608,36 +615,32 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	}
 	partial_flag = true;
 
-	simpb_side(equation, np, true, 2);
+	simpb_side(equation, np, true, true, 2);
 	debug_string(1, "Simplify result before applying polynomial operations:");
 	side_debug(1, equation, *np);
 	for (flag = false;;) {
 		/* divide top and bottom of fractions by any polynomial GCD found */
 		if (poly_gcd_simp(equation, np)) {
 			flag = false;
-			simpb_side(equation, np, true, 3);
+			simpb_side(equation, np, false, true, 3);
 		}
-		if (!flag) {
-			/* factor polynomials */
-			while (poly_factor(equation, np)) {
-				flag = true;
-				simpb_side(equation, np, true, 3);
-			}
-			if (flag) {
-				continue;
-			}
+		/* factor polynomials */
+		if (!flag && poly_factor(equation, np, true)) {
+			flag = true;
+			simpb_side(equation, np, false, true, 3);
+			continue;
 		}
 		/* simplify algebraic fractions with polynomial and smart division */
 		if (!frac_flag && div_remainder(equation, np, poly_flag, quick_flag)) {
 			flag = false;
-			simpb_side(equation, np, true, 3);
+			simpb_side(equation, np, false, true, 3);
 			continue;
 		}
 		break;
 	}
-	simpb_side(equation, np, true, 1);
+/*	simpb_side(equation, np, true, true, 1); */
 	simp_constant_power(equation, np);
-	simp_loop(equation, np);
+	simp_ssub(equation, np, 0L, 1.0, true, true, 1);
 	unsimp_power(equation, np);
 	make_fractions(equation, np);
 	factor_power(equation, np);
@@ -645,20 +648,15 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	make_fractions(equation, np);
 	uf_power(equation, np);
 	integer_root_simp(equation, np);
-	simpb_side(equation, np, true, 3);
-	if (!frac_flag) {
-		while (div_remainder(equation, np, poly_flag, quick_flag)) {
-			simpb_side(equation, np, true, 3);
-		}
-	}
-	poly_factor(equation, np);
-	simpb_side(equation, np, true, 2);
+	simpb_side(equation, np, true, true, 3);
+	poly_factor(equation, np, false);
+	simpb_side(equation, np, true, !frac_flag, 2);
 }
 
 /*
  * Commonly used quick simplify routine that doesn't factor.
  *
- * Return true if factor_times() simplified something.
+ * Return true if factor_times() did something.
  */
 int
 simp_loop(equation, np)
@@ -1671,8 +1669,8 @@ no_op2:
 			opa2[oc2] = pv1 + 1;
 			used[oc2] = false;
 			if (++oc2 >= ARR_CNT(opa2)) {
-				debug_string(1, "Expression too big to compare, because MAX_COMPARE_TERMS may be too small.");
-				return false;	/* expression too big to compare */
+				debug_string(1, "Expression too big to compare, because MAX_COMPARE_TERMS exceeded.");
+				return false;
 			}
 		}
 	}
@@ -2050,7 +2048,8 @@ int		i1, i2;
 }
 
 /*
- * Recursively check and possibly reorder a "level" subexpression which starts at "loc" in an equation side.
+ * Recursively check and possibly reorder a "level" subexpression
+ * which starts at "loc" in an equation side.
  */
 static int
 order_recurse(equation, np, loc, level)
@@ -2154,13 +2153,14 @@ int		*np, loc, level;
 	}
 	return modified;
 corrupt:
-	error_bug("Expression is corrupt!  Please send a bug report.");
+	error_bug("Internal representation of expression is corrupt!");
 	return modified;	/* not reached */
 }
 
 /*
  * Try to rationalize the denominator of algebraic fractions.
- * Only works with square roots and sometimes helps with their simplification.
+ * Only works with a square root in the denominator
+ * and sometimes helps with simplification.
  *
  * Returns true if something was done.
  * Unfactoring needs to be done immediately, if this returns true.
@@ -2228,13 +2228,13 @@ do_again:
 					    && equation[k].level == equation[k+1].level
 					    && equation[k+1].kind == CONSTANT
 					    && fmod(equation[k+1].token.constant, 1.0) == 0.5) {
-						if (count != 1) {
-							for (k1 = i + 2; k1 < end_loc; k1 += 2) {
-								if (equation[k1].token.operatr == POWER
-								    && equation[k1].level == equation[k1+1].level
-								    && equation[k1+1].kind == CONSTANT
-								    && fmod(equation[k1+1].token.constant, 1.0) == 0.5
-								    && k != k1) {
+						for (k1 = i + 2; k1 < end_loc; k1 += 2) {
+							if (equation[k1].token.operatr == POWER
+							    && equation[k1].level == equation[k1+1].level
+							    && equation[k1+1].kind == CONSTANT
+							    && fmod(equation[k1+1].token.constant, 1.0) == 0.5
+							    && k != k1) {
+								if (!(equation[k1].level == (div_level + 2) && count == 1)) {
 									i += 2;
 									goto be_thorough;
 								}
@@ -2272,8 +2272,9 @@ do_again:
 						equation[k1].token.operatr = DIVIDE;
 						k1++;
 						blt(&equation[k1], scratch, k * sizeof(token_type));
-						modified = true;
 						i = k1 + k;
+						debug_string(1, "Square roots in denominator rationalized.");
+						modified = true;
 						goto be_thorough;
 					}
 				}
