@@ -14,6 +14,7 @@ static int	quad_solve();
 static int	increase();
 
 static int	repeat_count;
+static int	last_int_var = 0;
 
 /*
  * Solve using equation spaces.
@@ -104,7 +105,7 @@ int		*rightnp;	/* pointer to length of RHS */
 		return false;
 	}
 	if (n_tlhs <= 0 || n_trhs <= 0) {
-		error(_("Please enter an equation or type \"help\"."));
+		error(_("Please enter an equation or a command like \"help\"."));
 		return false;
 	}
 	if (wantp->kind == VARIABLE) {
@@ -572,15 +573,17 @@ long	v;
 	partial_flag = false;
 	ufactor(tlhs, &n_tlhs);
 	partial_flag = true;
+	symb_flag = symblify;
 	simp_ssub(tlhs, &n_tlhs, v, d, true, false, 2);
-	simp_ssub(tlhs, &n_tlhs, 0L, /* 0.0 */ 1.0, true, true, 2);
+	simp_ssub(tlhs, &n_tlhs, 0L, 1.0, true, true, 2);
+	symb_flag = false;
 lonely:
 	ep = &tlhs[n_tlhs];
 	len2 = len1 = 0;
 	foundp = false;
 	for (p1 = tlhs + 1;; p1 += 2) {
 		if (p1 >= ep) {
-			return false;
+			return true;	/* power not found */
 		}
 		if (p1->level == 1) {
 			break;
@@ -982,7 +985,6 @@ big_bbreak:
 	len += alen;
 	if (found_var(scratch, len, v))
 		return false;
-	debug_string(1, _("Plugging equation into the quadratic formula:"));
 	blt(tlhs, x1_storage, nx1 * sizeof(token_type));
 	n_tlhs = nx1;
 	simp_loop(tlhs, &n_tlhs);
@@ -993,15 +995,7 @@ big_bbreak:
 	uf_tsimp(trhs, &n_trhs);	/* don't unfactor result so much, just unfactor what will be unfactored anyway */
 	simps_side(trhs, &n_trhs, false);
 	list_tdebug(1);
-#if	!SILENT
-	if (high_power == 2.0) {
-		fprintf(gfp, _("Equation was quadratic.\n"));
-	} else if (high_power == 4.0) {
-		fprintf(gfp, _("Equation was biquadratic.\n"));
-	} else {
-		fprintf(gfp, _("Equation was solved with the quadratic formula.\n"));
-	}
-#endif
+	debug_string(0, _("Equation was solved with the quadratic formula."));
 	return true;
 }
 
@@ -1029,6 +1023,8 @@ int		*side2np;	/* pointer to the length of "side2p" */
 	double		numerator, denominator;
 	static int	prev_n1, prev_n2;
 	double		d1, d2;
+	complexs	c1, c2;
+	char		var_name_buf[MAX_VAR_LEN];
 
 	oldn = *side1np;
 	ep = &side1p[oldn];
@@ -1068,16 +1064,35 @@ int		*side2np;	/* pointer to the length of "side2p" */
 	}
 	operandn = p1 - operandp;
 	if (op == POWER && operandp == side1p) {
-		if (!get_constant(side2p, *side2np, &d1))
+		if (!parse_complex(side2p, *side2np, &c1))
 			return false;
-		if (!get_constant(operandp, operandn, &d2))
+		if (!parse_complex(operandp, operandn, &c2))
 			return false;
 		debug_string(1, _("Taking logarithm of both equation sides:"));
-		*side2np = 1;
-		side2p->level = 1;
-		side2p->kind = CONSTANT;
 		errno = 0;
-		side2p->token.constant = log(d1) / log(d2);
+		c1 = complex_div(complex_log(c1), complex_log(c2));
+		*side2np = 0;
+		side2p[*side2np].level = 1;
+		side2p[*side2np].kind = CONSTANT;
+		side2p[*side2np].token.constant = c1.re;
+		(*side2np)++;
+		side2p[*side2np].level = 1;
+		side2p[*side2np].kind = OPERATOR;
+		side2p[*side2np].token.operatr = PLUS;
+		(*side2np)++;
+		side2p[*side2np].level = 2;
+		side2p[*side2np].kind = CONSTANT;
+		side2p[*side2np].token.constant = c1.im;
+		(*side2np)++;
+		side2p[*side2np].level = 2;
+		side2p[*side2np].kind = OPERATOR;
+		side2p[*side2np].token.operatr = TIMES;
+		(*side2np)++;
+		side2p[*side2np].level = 2;
+		side2p[*side2np].kind = VARIABLE;
+		side2p[*side2np].token.variable = IMAGINARY;
+		(*side2np)++;
+
 		blt(side1p, p1 + 1, (*side1np - (operandn + 1)) * sizeof(token_type));
 		*side1np -= operandn + 1;
 		check_err();
@@ -1158,7 +1173,12 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		p2++;
 		p2->level = 2;
 		p2->kind = VARIABLE;
-		parse_var(&p2->token.variable, V_INTEGER_NAME);
+		snprintf(var_name_buf, sizeof(var_name_buf), "%s%.0d", V_INTEGER_NAME, last_int_var);
+		parse_var(&p2->token.variable, var_name_buf);
+		last_int_var++;
+		if (last_int_var < 0) {
+			last_int_var = 0;
+		}
 		p2++;
 		p2->level = 2;
 		p2->kind = OPERATOR;
