@@ -67,9 +67,11 @@ int		level;		/* current level of parentheses within the subexpression starting a
 	if (group_flag) {
 		for (i = di + 1; i < edi; i++) {
 			if (equation[i].level == level && equation[i].kind == OPERATOR) {
+#if	false
 				if (equation[i].token.operatr != DIVIDE) {
 					error_bug("Bug in group_recurse().");
 				}
+#endif
 				equation[i].token.operatr = TIMES;
 			}
 			equation[i].level++;
@@ -92,52 +94,68 @@ int		*np;		/* pointer to length of equation side */
 
 /*
  * Make equation side ready for display.
+ *
+ * Return true if any fractions were created.
  */
-void
+int
 display_fractions_and_group(equation, np)
 token_type	*equation;
 int		*np;
 {
+	int	rv;
+
 	elim_loop(equation, np);
-	make_fractions(equation, np);
+	rv = make_fractions(equation, np);
 	group_proc(equation, np);
+	return rv;
 }
 
 /*
  * This function is the guts of the display command.
+ *
+ * Return true if any fractions were created.
  */
-void
+int
 make_fractions_and_group(n)
 int	n;	/* equation space number */
 {
+	int	rv;
+
 	if (n_lhs[n] <= 0)
-		return;
-	display_fractions_and_group(lhs[n], &n_lhs[n]);
+		return false;
+	rv = display_fractions_and_group(lhs[n], &n_lhs[n]);
 	if (n_rhs[n]) {
-		display_fractions_and_group(rhs[n], &n_rhs[n]);
+		if (display_fractions_and_group(rhs[n], &n_rhs[n]))
+			rv = true;
 	}
+	return rv;
 }
 
 /*
- * Combine fractions by making the denominators all the same.
- * This means converting "a/b+c/d" to "(a*d+c*b)/b/d".
+ * Efficiently combine algebraic fractions added together
+ * by putting all terms over a common denominator.
+ * This means converting "(a/b)+(c/d)+f" directly to "(a*d+c*b+b*d*f)/b/d".
+ * The resulting expression is always equivalent to the original expression.
  *
  * If start_flag is 0, only combine denominators to convert complex fractions to simple fractions.
  * Level one addition of fractions will be unchanged.
+ * Can make an expression over-complicated.
  *
  * If start_flag is 1, always combine denominators.
  * This can easily make an expression large and complicated.
+ * Used when solving for zero.
  *
- * If start_flag is 2, always combine denominators and remove any polynomial GCD between them.
+ * If start_flag is 2, always combine denominators
+ * and reduce the result by removing any polynomial GCD between them.
  * Note that this wipes out globals tlhs[] and trhs[].
  * This simplifies all algebraic fractions into a single simple fraction
- * and prevents making a more complicated (or larger) algebraic fraction.
+ * and usually prevents making a more complicated (or larger) algebraic fraction.
  *
- * Return true if equation side was modified.
+ * Return true if the equation side was modified.
  */
 int
 super_factor(equation, np, start_flag)
-token_type	*equation;	/* pointer to the beginning of the equation side */
+token_type	*equation;	/* pointer to the beginning of the equation side to process */
 int		*np;		/* pointer to the length of the equation side */
 int		start_flag;
 {
@@ -161,11 +179,7 @@ int		*np, loc, level, start_flag;
 
 	if (!start_flag) {
 		for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
-#if	true
 			if (equation[i].level == level && equation[i].token.operatr == DIVIDE) {
-#else
-			if (equation[i].level == level && (equation[i].token.operatr == DIVIDE || equation[i].token.operatr == POWER)) {
-#endif
 				start_flag = true;
 				break;
 			}
